# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CarlaSettingsW(object):
    def setupUi(self, CarlaSettingsW):
        CarlaSettingsW.setObjectName("CarlaSettingsW")
        CarlaSettingsW.resize(612, 620)
        CarlaSettingsW.setMinimumSize(QtCore.QSize(600, 0))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(CarlaSettingsW)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lw_page = QtWidgets.QTableWidget(CarlaSettingsW)
        self.lw_page.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.lw_page.setProperty("showDropIndicator", False)
        self.lw_page.setDragDropOverwriteMode(False)
        self.lw_page.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.lw_page.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.lw_page.setIconSize(QtCore.QSize(48, 48))
        self.lw_page.setTextElideMode(QtCore.Qt.ElideMiddle)
        self.lw_page.setWordWrap(False)
        self.lw_page.setCornerButtonEnabled(False)
        self.lw_page.setObjectName("lw_page")
        self.lw_page.setColumnCount(1)
        self.lw_page.setRowCount(8)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setVerticalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.lw_page.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/scalable/carla.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(0, 0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/scalable/canvas.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon1)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(1, 0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/48x48/jack.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon2)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(2, 0, item)
        item = QtWidgets.QTableWidgetItem()
        item.setText("OSC")
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/scalable/osc.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon3)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(3, 0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/scalable/folder.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon4)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(4, 0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        item.setIcon(icon4)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(5, 0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/scalable/wine.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon5)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(6, 0, item)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/scalable/warning.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon6)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.lw_page.setItem(7, 0, item)
        self.lw_page.horizontalHeader().setVisible(False)
        self.lw_page.horizontalHeader().setStretchLastSection(True)
        self.lw_page.verticalHeader().setVisible(False)
        self.lw_page.verticalHeader().setDefaultSectionSize(52)
        self.horizontalLayout_3.addWidget(self.lw_page)
        self.stackedWidget = QtWidgets.QStackedWidget(CarlaSettingsW)
        self.stackedWidget.setLineWidth(0)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_main = QtWidgets.QWidget()
        self.page_main.setObjectName("page_main")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.page_main)
        self.verticalLayout_5.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_main = QtWidgets.QLabel(self.page_main)
        self.label_main.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_main.setObjectName("label_main")
        self.horizontalLayout.addWidget(self.label_main)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label_icon_main = QtWidgets.QLabel(self.page_main)
        self.label_icon_main.setMaximumSize(QtCore.QSize(48, 48))
        self.label_icon_main.setText("")
        self.label_icon_main.setPixmap(QtGui.QPixmap(":/scalable/carla.svg"))
        self.label_icon_main.setScaledContents(True)
        self.label_icon_main.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_icon_main.setObjectName("label_icon_main")
        self.horizontalLayout.addWidget(self.label_icon_main)
        self.verticalLayout_5.addLayout(self.horizontalLayout)
        self.group_main_paths = QtWidgets.QGroupBox(self.page_main)
        self.group_main_paths.setObjectName("group_main_paths")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.group_main_paths)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_main_proj_folder_open = QtWidgets.QLabel(self.group_main_paths)
        self.label_main_proj_folder_open.setObjectName("label_main_proj_folder_open")
        self.horizontalLayout_2.addWidget(self.label_main_proj_folder_open)
        self.le_main_proj_folder = QtWidgets.QLineEdit(self.group_main_paths)
        self.le_main_proj_folder.setObjectName("le_main_proj_folder")
        self.horizontalLayout_2.addWidget(self.le_main_proj_folder)
        self.b_main_proj_folder_open = QtWidgets.QPushButton(self.group_main_paths)
        self.b_main_proj_folder_open.setMaximumSize(QtCore.QSize(22, 22))
        self.b_main_proj_folder_open.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/16x16/document-open.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_main_proj_folder_open.setIcon(icon7)
        self.b_main_proj_folder_open.setIconSize(QtCore.QSize(16, 16))
        self.b_main_proj_folder_open.setObjectName("b_main_proj_folder_open")
        self.horizontalLayout_2.addWidget(self.b_main_proj_folder_open)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout_5.addWidget(self.group_main_paths)
        self.group_main_misc = QtWidgets.QGroupBox(self.page_main)
        self.group_main_misc.setObjectName("group_main_misc")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.group_main_misc)
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem1 = QtWidgets.QSpacerItem(130, 10, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem1, 4, 2, 1, 1)
        self.cb_main_classic_skin_default = QtWidgets.QCheckBox(self.group_main_misc)
        self.cb_main_classic_skin_default.setObjectName("cb_main_classic_skin_default")
        self.gridLayout_4.addWidget(self.cb_main_classic_skin_default, 3, 0, 1, 3)
        self.label_main_refresh_interval = QtWidgets.QLabel(self.group_main_misc)
        self.label_main_refresh_interval.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_main_refresh_interval.setObjectName("label_main_refresh_interval")
        self.gridLayout_4.addWidget(self.label_main_refresh_interval, 4, 0, 1, 1)
        self.sb_main_refresh_interval = QtWidgets.QSpinBox(self.group_main_misc)
        self.sb_main_refresh_interval.setMinimum(10)
        self.sb_main_refresh_interval.setMaximum(1000)
        self.sb_main_refresh_interval.setProperty("value", 250)
        self.sb_main_refresh_interval.setObjectName("sb_main_refresh_interval")
        self.gridLayout_4.addWidget(self.sb_main_refresh_interval, 4, 1, 1, 1)
        self.ch_main_show_logs = QtWidgets.QCheckBox(self.group_main_misc)
        self.ch_main_show_logs.setObjectName("ch_main_show_logs")
        self.gridLayout_4.addWidget(self.ch_main_show_logs, 1, 0, 1, 3)
        self.ch_main_confirm_exit = QtWidgets.QCheckBox(self.group_main_misc)
        self.ch_main_confirm_exit.setObjectName("ch_main_confirm_exit")
        self.gridLayout_4.addWidget(self.ch_main_confirm_exit, 0, 0, 1, 3)
        self.verticalLayout_5.addWidget(self.group_main_misc)
        self.group_main_theme = QtWidgets.QGroupBox(self.page_main)
        self.group_main_theme.setObjectName("group_main_theme")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.group_main_theme)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.ch_main_theme_pro = QtWidgets.QCheckBox(self.group_main_theme)
        self.ch_main_theme_pro.setObjectName("ch_main_theme_pro")
        self.gridLayout_3.addWidget(self.ch_main_theme_pro, 0, 0, 1, 3)
        self.label_main_theme_color = QtWidgets.QLabel(self.group_main_theme)
        self.label_main_theme_color.setEnabled(False)
        self.label_main_theme_color.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_main_theme_color.setObjectName("label_main_theme_color")
        self.gridLayout_3.addWidget(self.label_main_theme_color, 1, 0, 1, 1)
        self.cb_main_theme_color = QtWidgets.QComboBox(self.group_main_theme)
        self.cb_main_theme_color.setEnabled(False)
        self.cb_main_theme_color.setObjectName("cb_main_theme_color")
        self.cb_main_theme_color.addItem("")
        self.cb_main_theme_color.addItem("")
        self.gridLayout_3.addWidget(self.cb_main_theme_color, 1, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(130, 10, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem2, 1, 2, 1, 1)
        self.verticalLayout_5.addWidget(self.group_main_theme)
        self.group_main_experimental = QtWidgets.QGroupBox(self.page_main)
        self.group_main_experimental.setObjectName("group_main_experimental")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.group_main_experimental)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.ch_main_experimental = QtWidgets.QCheckBox(self.group_main_experimental)
        self.ch_main_experimental.setObjectName("ch_main_experimental")
        self.verticalLayout_10.addWidget(self.ch_main_experimental)
        self.verticalLayout_5.addWidget(self.group_main_experimental)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem3)
        self.stackedWidget.addWidget(self.page_main)
        self.page_canvas = QtWidgets.QWidget()
        self.page_canvas.setObjectName("page_canvas")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.page_canvas)
        self.verticalLayout_8.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_9 = QtWidgets.QLabel(self.page_canvas)
        self.label_9.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_10.addWidget(self.label_9)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem4)
        self.label_10 = QtWidgets.QLabel(self.page_canvas)
        self.label_10.setMaximumSize(QtCore.QSize(48, 48))
        self.label_10.setText("")
        self.label_10.setPixmap(QtGui.QPixmap(":/scalable/canvas.svg"))
        self.label_10.setScaledContents(True)
        self.label_10.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_10.addWidget(self.label_10)
        self.verticalLayout_8.addLayout(self.horizontalLayout_10)
        self.group_canvas_theme = QtWidgets.QGroupBox(self.page_canvas)
        self.group_canvas_theme.setObjectName("group_canvas_theme")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.group_canvas_theme)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem5, 0, 4, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(80, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem6, 2, 0, 1, 2)
        self.cb_canvas_bezier_lines = QtWidgets.QCheckBox(self.group_canvas_theme)
        self.cb_canvas_bezier_lines.setObjectName("cb_canvas_bezier_lines")
        self.gridLayout_2.addWidget(self.cb_canvas_bezier_lines, 2, 2, 1, 3)
        self.cb_canvas_theme = QtWidgets.QComboBox(self.group_canvas_theme)
        self.cb_canvas_theme.setObjectName("cb_canvas_theme")
        self.gridLayout_2.addWidget(self.cb_canvas_theme, 0, 3, 1, 1)
        self.label_canvas_theme = QtWidgets.QLabel(self.group_canvas_theme)
        self.label_canvas_theme.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_canvas_theme.setObjectName("label_canvas_theme")
        self.gridLayout_2.addWidget(self.label_canvas_theme, 0, 1, 1, 2)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem7, 0, 0, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem8, 1, 0, 1, 1)
        self.label_canvas_size = QtWidgets.QLabel(self.group_canvas_theme)
        self.label_canvas_size.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_canvas_size.setObjectName("label_canvas_size")
        self.gridLayout_2.addWidget(self.label_canvas_size, 1, 1, 1, 2)
        self.cb_canvas_size = QtWidgets.QComboBox(self.group_canvas_theme)
        self.cb_canvas_size.setObjectName("cb_canvas_size")
        self.cb_canvas_size.addItem("")
        self.cb_canvas_size.addItem("")
        self.cb_canvas_size.addItem("")
        self.cb_canvas_size.addItem("")
        self.cb_canvas_size.addItem("")
        self.cb_canvas_size.addItem("")
        self.gridLayout_2.addWidget(self.cb_canvas_size, 1, 3, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem9, 1, 4, 1, 1)
        self.verticalLayout_8.addWidget(self.group_canvas_theme)
        self.group_canvas_options = QtWidgets.QGroupBox(self.page_canvas)
        self.group_canvas_options.setObjectName("group_canvas_options")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.group_canvas_options)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.cb_canvas_hide_groups = QtWidgets.QCheckBox(self.group_canvas_options)
        self.cb_canvas_hide_groups.setObjectName("cb_canvas_hide_groups")
        self.verticalLayout_7.addWidget(self.cb_canvas_hide_groups)
        self.cb_canvas_auto_select = QtWidgets.QCheckBox(self.group_canvas_options)
        self.cb_canvas_auto_select.setObjectName("cb_canvas_auto_select")
        self.verticalLayout_7.addWidget(self.cb_canvas_auto_select)
        self.cb_canvas_eyecandy = QtWidgets.QCheckBox(self.group_canvas_options)
        self.cb_canvas_eyecandy.setObjectName("cb_canvas_eyecandy")
        self.verticalLayout_7.addWidget(self.cb_canvas_eyecandy)
        self.verticalLayout_8.addWidget(self.group_canvas_options)
        self.group_canvas_render = QtWidgets.QGroupBox(self.page_canvas)
        self.group_canvas_render.setObjectName("group_canvas_render")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.group_canvas_render)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.cb_canvas_render_aa = QtWidgets.QCheckBox(self.group_canvas_render)
        self.cb_canvas_render_aa.setTristate(True)
        self.cb_canvas_render_aa.setObjectName("cb_canvas_render_aa")
        self.verticalLayout_9.addWidget(self.cb_canvas_render_aa)
        self.cb_canvas_full_repaints = QtWidgets.QCheckBox(self.group_canvas_render)
        self.cb_canvas_full_repaints.setObjectName("cb_canvas_full_repaints")
        self.verticalLayout_9.addWidget(self.cb_canvas_full_repaints)
        self.verticalLayout_8.addWidget(self.group_canvas_render)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem10)
        self.stackedWidget.addWidget(self.page_canvas)
        self.page_engine = QtWidgets.QWidget()
        self.page_engine.setObjectName("page_engine")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.page_engine)
        self.verticalLayout_6.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_6 = QtWidgets.QLabel(self.page_engine)
        self.label_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_5.addWidget(self.label_6)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem11)
        self.label_icon_engine = QtWidgets.QLabel(self.page_engine)
        self.label_icon_engine.setText("")
        self.label_icon_engine.setPixmap(QtGui.QPixmap(":/48x48/jack.png"))
        self.label_icon_engine.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_icon_engine.setObjectName("label_icon_engine")
        self.horizontalLayout_5.addWidget(self.label_icon_engine)
        self.verticalLayout_6.addLayout(self.horizontalLayout_5)
        self.groupBox = QtWidgets.QGroupBox(self.page_engine)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.sw_engine_process_mode = QtWidgets.QStackedWidget(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sw_engine_process_mode.sizePolicy().hasHeightForWidth())
        self.sw_engine_process_mode.setSizePolicy(sizePolicy)
        self.sw_engine_process_mode.setObjectName("sw_engine_process_mode")
        self.page_engine_process_mode_jack = QtWidgets.QWidget()
        self.page_engine_process_mode_jack.setObjectName("page_engine_process_mode_jack")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.page_engine_process_mode_jack)
        self.horizontalLayout_4.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.cb_engine_process_mode_jack = QtWidgets.QComboBox(self.page_engine_process_mode_jack)
        self.cb_engine_process_mode_jack.setObjectName("cb_engine_process_mode_jack")
        self.cb_engine_process_mode_jack.addItem("")
        self.cb_engine_process_mode_jack.addItem("")
        self.cb_engine_process_mode_jack.addItem("")
        self.cb_engine_process_mode_jack.addItem("")
        self.horizontalLayout_4.addWidget(self.cb_engine_process_mode_jack)
        self.sw_engine_process_mode.addWidget(self.page_engine_process_mode_jack)
        self.page_engine_process_mode_other = QtWidgets.QWidget()
        self.page_engine_process_mode_other.setObjectName("page_engine_process_mode_other")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.page_engine_process_mode_other)
        self.horizontalLayout_7.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.cb_engine_process_mode_other = QtWidgets.QComboBox(self.page_engine_process_mode_other)
        self.cb_engine_process_mode_other.setObjectName("cb_engine_process_mode_other")
        self.cb_engine_process_mode_other.addItem("")
        self.cb_engine_process_mode_other.addItem("")
        self.horizontalLayout_7.addWidget(self.cb_engine_process_mode_other)
        self.sw_engine_process_mode.addWidget(self.page_engine_process_mode_other)
        self.gridLayout_7.addWidget(self.sw_engine_process_mode, 1, 2, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.groupBox)
        self.label_21.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_21.setObjectName("label_21")
        self.gridLayout_7.addWidget(self.label_21, 0, 1, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.groupBox)
        self.label_20.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_20.setObjectName("label_20")
        self.gridLayout_7.addWidget(self.label_20, 1, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout_7.addWidget(self.label_7, 2, 1, 1, 1)
        self.sb_engine_max_params = QtWidgets.QSpinBox(self.groupBox)
        self.sb_engine_max_params.setMaximum(1000)
        self.sb_engine_max_params.setObjectName("sb_engine_max_params")
        self.gridLayout_7.addWidget(self.sb_engine_max_params, 2, 2, 1, 1)
        self.tb_engine_driver_config = QtWidgets.QToolButton(self.groupBox)
        self.tb_engine_driver_config.setObjectName("tb_engine_driver_config")
        self.gridLayout_7.addWidget(self.tb_engine_driver_config, 0, 3, 1, 1)
        self.cb_engine_audio_driver = QtWidgets.QComboBox(self.groupBox)
        self.cb_engine_audio_driver.setObjectName("cb_engine_audio_driver")
        self.gridLayout_7.addWidget(self.cb_engine_audio_driver, 0, 2, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem12, 0, 0, 1, 1)
        spacerItem13 = QtWidgets.QSpacerItem(100, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem13, 1, 3, 1, 1)
        self.cb_engine_reset_xruns = QtWidgets.QCheckBox(self.groupBox)
        self.cb_engine_reset_xruns.setObjectName("cb_engine_reset_xruns")
        self.gridLayout_7.addWidget(self.cb_engine_reset_xruns, 3, 0, 1, 4)
        self.verticalLayout_6.addWidget(self.groupBox)
        self.groupBox_8 = QtWidgets.QGroupBox(self.page_engine)
        self.groupBox_8.setObjectName("groupBox_8")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_8)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.label_engine_ui_bridges_timeout = QtWidgets.QLabel(self.groupBox_8)
        self.label_engine_ui_bridges_timeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_engine_ui_bridges_timeout.setObjectName("label_engine_ui_bridges_timeout")
        self.gridLayout_8.addWidget(self.label_engine_ui_bridges_timeout, 5, 0, 1, 1)
        self.sb_engine_ui_bridges_timeout = QtWidgets.QSpinBox(self.groupBox_8)
        self.sb_engine_ui_bridges_timeout.setMaximum(10000)
        self.sb_engine_ui_bridges_timeout.setObjectName("sb_engine_ui_bridges_timeout")
        self.gridLayout_8.addWidget(self.sb_engine_ui_bridges_timeout, 5, 1, 1, 1)
        spacerItem14 = QtWidgets.QSpacerItem(100, 10, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_8.addItem(spacerItem14, 5, 2, 1, 1)
        self.ch_engine_prefer_ui_bridges = QtWidgets.QCheckBox(self.groupBox_8)
        self.ch_engine_prefer_ui_bridges.setObjectName("ch_engine_prefer_ui_bridges")
        self.gridLayout_8.addWidget(self.ch_engine_prefer_ui_bridges, 2, 0, 1, 3)
        self.ch_engine_uis_always_on_top = QtWidgets.QCheckBox(self.groupBox_8)
        self.ch_engine_uis_always_on_top.setObjectName("ch_engine_uis_always_on_top")
        self.gridLayout_8.addWidget(self.ch_engine_uis_always_on_top, 1, 0, 1, 3)
        self.ch_engine_manage_uis = QtWidgets.QCheckBox(self.groupBox_8)
        self.ch_engine_manage_uis.setObjectName("ch_engine_manage_uis")
        self.gridLayout_8.addWidget(self.ch_engine_manage_uis, 0, 0, 1, 3)
        self.label_engine_ui_bridges_mac_note = QtWidgets.QLabel(self.groupBox_8)
        self.label_engine_ui_bridges_mac_note.setObjectName("label_engine_ui_bridges_mac_note")
        self.gridLayout_8.addWidget(self.label_engine_ui_bridges_mac_note, 6, 0, 1, 3)
        self.verticalLayout_6.addWidget(self.groupBox_8)
        self.label_15 = QtWidgets.QLabel(self.page_engine)
        self.label_15.setMaximumSize(QtCore.QSize(16777215, 5))
        self.label_15.setText("")
        self.label_15.setObjectName("label_15")
        self.verticalLayout_6.addWidget(self.label_15)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem15)
        self.label_22 = QtWidgets.QLabel(self.page_engine)
        self.label_22.setMaximumSize(QtCore.QSize(22, 22))
        self.label_22.setText("")
        self.label_22.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_22.setScaledContents(True)
        self.label_22.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_22.setObjectName("label_22")
        self.horizontalLayout_13.addWidget(self.label_22)
        self.label_23 = QtWidgets.QLabel(self.page_engine)
        self.label_23.setObjectName("label_23")
        self.horizontalLayout_13.addWidget(self.label_23)
        spacerItem16 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem16)
        self.verticalLayout_6.addLayout(self.horizontalLayout_13)
        spacerItem17 = QtWidgets.QSpacerItem(20, 300, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem17)
        self.stackedWidget.addWidget(self.page_engine)
        self.page_osc = QtWidgets.QWidget()
        self.page_osc.setObjectName("page_osc")
        self.verticalLayout_22 = QtWidgets.QVBoxLayout(self.page_osc)
        self.verticalLayout_22.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_22.setObjectName("verticalLayout_22")
        self.horizontalLayout_22 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_22.setObjectName("horizontalLayout_22")
        self.label_14 = QtWidgets.QLabel(self.page_osc)
        self.label_14.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_22.addWidget(self.label_14)
        spacerItem18 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_22.addItem(spacerItem18)
        self.label_icon_engine_5 = QtWidgets.QLabel(self.page_osc)
        self.label_icon_engine_5.setMaximumSize(QtCore.QSize(48, 48))
        self.label_icon_engine_5.setText("")
        self.label_icon_engine_5.setPixmap(QtGui.QPixmap(":/scalable/osc.svgz"))
        self.label_icon_engine_5.setScaledContents(True)
        self.label_icon_engine_5.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_icon_engine_5.setObjectName("label_icon_engine_5")
        self.horizontalLayout_22.addWidget(self.label_icon_engine_5)
        self.verticalLayout_22.addLayout(self.horizontalLayout_22)
        self.group_osc_core = QtWidgets.QGroupBox(self.page_osc)
        self.group_osc_core.setObjectName("group_osc_core")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.group_osc_core)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.ch_osc_enable = QtWidgets.QCheckBox(self.group_osc_core)
        self.ch_osc_enable.setObjectName("ch_osc_enable")
        self.gridLayout_13.addWidget(self.ch_osc_enable, 0, 1, 1, 1)
        spacerItem19 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_13.addItem(spacerItem19, 0, 0, 1, 1)
        spacerItem20 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_13.addItem(spacerItem20, 0, 2, 1, 1)
        self.verticalLayout_22.addWidget(self.group_osc_core)
        self.group_osc_tcp_port = QtWidgets.QGroupBox(self.page_osc)
        self.group_osc_tcp_port.setCheckable(True)
        self.group_osc_tcp_port.setObjectName("group_osc_tcp_port")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.group_osc_tcp_port)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.rb_osc_tcp_port_specific = QtWidgets.QRadioButton(self.group_osc_tcp_port)
        self.rb_osc_tcp_port_specific.setObjectName("rb_osc_tcp_port_specific")
        self.gridLayout_12.addWidget(self.rb_osc_tcp_port_specific, 1, 1, 1, 1)
        self.horizontalLayout_24 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_24.setObjectName("horizontalLayout_24")
        spacerItem21 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_24.addItem(spacerItem21)
        self.label_39 = QtWidgets.QLabel(self.group_osc_tcp_port)
        self.label_39.setMaximumSize(QtCore.QSize(22, 22))
        self.label_39.setText("")
        self.label_39.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_39.setScaledContents(True)
        self.label_39.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_39.setObjectName("label_39")
        self.horizontalLayout_24.addWidget(self.label_39)
        self.label_40 = QtWidgets.QLabel(self.group_osc_tcp_port)
        self.label_40.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.label_40.setObjectName("label_40")
        self.horizontalLayout_24.addWidget(self.label_40)
        spacerItem22 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_24.addItem(spacerItem22)
        self.gridLayout_12.addLayout(self.horizontalLayout_24, 2, 0, 1, 4)
        spacerItem23 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_12.addItem(spacerItem23, 0, 0, 2, 1)
        spacerItem24 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_12.addItem(spacerItem24, 0, 3, 2, 1)
        self.sb_osc_tcp_port_number = QtWidgets.QSpinBox(self.group_osc_tcp_port)
        self.sb_osc_tcp_port_number.setEnabled(False)
        self.sb_osc_tcp_port_number.setMinimum(1024)
        self.sb_osc_tcp_port_number.setMaximum(32767)
        self.sb_osc_tcp_port_number.setObjectName("sb_osc_tcp_port_number")
        self.gridLayout_12.addWidget(self.sb_osc_tcp_port_number, 1, 2, 1, 1)
        self.rb_osc_tcp_port_random = QtWidgets.QRadioButton(self.group_osc_tcp_port)
        self.rb_osc_tcp_port_random.setChecked(True)
        self.rb_osc_tcp_port_random.setObjectName("rb_osc_tcp_port_random")
        self.gridLayout_12.addWidget(self.rb_osc_tcp_port_random, 0, 1, 1, 2)
        self.verticalLayout_22.addWidget(self.group_osc_tcp_port)
        self.group_osc_udp_port = QtWidgets.QGroupBox(self.page_osc)
        self.group_osc_udp_port.setCheckable(True)
        self.group_osc_udp_port.setObjectName("group_osc_udp_port")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.group_osc_udp_port)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.rb_osc_udp_port_specific = QtWidgets.QRadioButton(self.group_osc_udp_port)
        self.rb_osc_udp_port_specific.setObjectName("rb_osc_udp_port_specific")
        self.gridLayout_11.addWidget(self.rb_osc_udp_port_specific, 1, 1, 1, 1)
        spacerItem25 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_11.addItem(spacerItem25, 0, 0, 2, 1)
        spacerItem26 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_11.addItem(spacerItem26, 0, 3, 2, 1)
        self.sb_osc_udp_port_number = QtWidgets.QSpinBox(self.group_osc_udp_port)
        self.sb_osc_udp_port_number.setEnabled(False)
        self.sb_osc_udp_port_number.setMinimum(1024)
        self.sb_osc_udp_port_number.setMaximum(32767)
        self.sb_osc_udp_port_number.setObjectName("sb_osc_udp_port_number")
        self.gridLayout_11.addWidget(self.sb_osc_udp_port_number, 1, 2, 1, 1)
        self.rb_osc_udp_port_random = QtWidgets.QRadioButton(self.group_osc_udp_port)
        self.rb_osc_udp_port_random.setChecked(True)
        self.rb_osc_udp_port_random.setObjectName("rb_osc_udp_port_random")
        self.gridLayout_11.addWidget(self.rb_osc_udp_port_random, 0, 1, 1, 2)
        self.horizontalLayout_25 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_25.setObjectName("horizontalLayout_25")
        spacerItem27 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_25.addItem(spacerItem27)
        self.label_41 = QtWidgets.QLabel(self.group_osc_udp_port)
        self.label_41.setMaximumSize(QtCore.QSize(22, 22))
        self.label_41.setText("")
        self.label_41.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_41.setScaledContents(True)
        self.label_41.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_41.setObjectName("label_41")
        self.horizontalLayout_25.addWidget(self.label_41)
        self.label_42 = QtWidgets.QLabel(self.group_osc_udp_port)
        self.label_42.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.label_42.setObjectName("label_42")
        self.horizontalLayout_25.addWidget(self.label_42)
        spacerItem28 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_25.addItem(spacerItem28)
        self.gridLayout_11.addLayout(self.horizontalLayout_25, 2, 0, 1, 4)
        self.verticalLayout_22.addWidget(self.group_osc_udp_port)
        self.label_36 = QtWidgets.QLabel(self.page_osc)
        self.label_36.setMaximumSize(QtCore.QSize(16777215, 5))
        self.label_36.setText("")
        self.label_36.setObjectName("label_36")
        self.verticalLayout_22.addWidget(self.label_36)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        spacerItem29 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem29)
        self.label_32 = QtWidgets.QLabel(self.page_osc)
        self.label_32.setMaximumSize(QtCore.QSize(22, 22))
        self.label_32.setText("")
        self.label_32.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_32.setScaledContents(True)
        self.label_32.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_32.setObjectName("label_32")
        self.horizontalLayout_21.addWidget(self.label_32)
        self.label_35 = QtWidgets.QLabel(self.page_osc)
        self.label_35.setObjectName("label_35")
        self.horizontalLayout_21.addWidget(self.label_35)
        spacerItem30 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem30)
        self.verticalLayout_22.addLayout(self.horizontalLayout_21)
        self.horizontalLayout_23 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_23.setObjectName("horizontalLayout_23")
        spacerItem31 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_23.addItem(spacerItem31)
        self.label_37 = QtWidgets.QLabel(self.page_osc)
        self.label_37.setMaximumSize(QtCore.QSize(22, 22))
        self.label_37.setText("")
        self.label_37.setPixmap(QtGui.QPixmap(":/16x16/dialog-warning.svgz"))
        self.label_37.setScaledContents(True)
        self.label_37.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_37.setObjectName("label_37")
        self.horizontalLayout_23.addWidget(self.label_37)
        self.label_38 = QtWidgets.QLabel(self.page_osc)
        self.label_38.setObjectName("label_38")
        self.horizontalLayout_23.addWidget(self.label_38)
        spacerItem32 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_23.addItem(spacerItem32)
        self.verticalLayout_22.addLayout(self.horizontalLayout_23)
        spacerItem33 = QtWidgets.QSpacerItem(20, 126, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_22.addItem(spacerItem33)
        self.label_36.raise_()
        self.group_osc_tcp_port.raise_()
        self.group_osc_udp_port.raise_()
        self.group_osc_core.raise_()
        self.stackedWidget.addWidget(self.page_osc)
        self.page_filepaths = QtWidgets.QWidget()
        self.page_filepaths.setObjectName("page_filepaths")
        self.verticalLayout_30 = QtWidgets.QVBoxLayout(self.page_filepaths)
        self.verticalLayout_30.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_30.setObjectName("verticalLayout_30")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_24 = QtWidgets.QLabel(self.page_filepaths)
        self.label_24.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_24.setObjectName("label_24")
        self.horizontalLayout_8.addWidget(self.label_24)
        spacerItem34 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem34)
        self.label_31 = QtWidgets.QLabel(self.page_filepaths)
        self.label_31.setMaximumSize(QtCore.QSize(48, 48))
        self.label_31.setText("")
        self.label_31.setPixmap(QtGui.QPixmap(":/scalable/folder.svgz"))
        self.label_31.setScaledContents(True)
        self.label_31.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_31.setWordWrap(True)
        self.label_31.setObjectName("label_31")
        self.horizontalLayout_8.addWidget(self.label_31)
        self.verticalLayout_30.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.cb_filepaths = QtWidgets.QComboBox(self.page_filepaths)
        self.cb_filepaths.setMinimumSize(QtCore.QSize(120, 0))
        self.cb_filepaths.setObjectName("cb_filepaths")
        self.cb_filepaths.addItem("")
        self.cb_filepaths.addItem("")
        self.horizontalLayout_19.addWidget(self.cb_filepaths)
        self.tw_filepaths_info = QtWidgets.QStackedWidget(self.page_filepaths)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tw_filepaths_info.sizePolicy().hasHeightForWidth())
        self.tw_filepaths_info.setSizePolicy(sizePolicy)
        self.tw_filepaths_info.setObjectName("tw_filepaths_info")
        self.tw_filepaths_info_audio = QtWidgets.QWidget()
        self.tw_filepaths_info_audio.setObjectName("tw_filepaths_info_audio")
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout(self.tw_filepaths_info_audio)
        self.horizontalLayout_20.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_20.setSpacing(0)
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.label_12 = QtWidgets.QLabel(self.tw_filepaths_info_audio)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_20.addWidget(self.label_12)
        self.tw_filepaths_info.addWidget(self.tw_filepaths_info_audio)
        self.tw_filepaths_info_midi = QtWidgets.QWidget()
        self.tw_filepaths_info_midi.setObjectName("tw_filepaths_info_midi")
        self.horizontalLayout_26 = QtWidgets.QHBoxLayout(self.tw_filepaths_info_midi)
        self.horizontalLayout_26.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_26.setSpacing(0)
        self.horizontalLayout_26.setObjectName("horizontalLayout_26")
        self.label_13 = QtWidgets.QLabel(self.tw_filepaths_info_midi)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_26.addWidget(self.label_13)
        self.tw_filepaths_info.addWidget(self.tw_filepaths_info_midi)
        self.horizontalLayout_19.addWidget(self.tw_filepaths_info)
        spacerItem35 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem35)
        self.verticalLayout_30.addLayout(self.horizontalLayout_19)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.tw_filepaths = QtWidgets.QStackedWidget(self.page_filepaths)
        self.tw_filepaths.setObjectName("tw_filepaths")
        self.tw_filepaths_audio = QtWidgets.QWidget()
        self.tw_filepaths_audio.setObjectName("tw_filepaths_audio")
        self.verticalLayout_18 = QtWidgets.QVBoxLayout(self.tw_filepaths_audio)
        self.verticalLayout_18.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_18.setSpacing(0)
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        self.lw_files_audio = QtWidgets.QListWidget(self.tw_filepaths_audio)
        self.lw_files_audio.setObjectName("lw_files_audio")
        self.verticalLayout_18.addWidget(self.lw_files_audio)
        self.tw_filepaths.addWidget(self.tw_filepaths_audio)
        self.tw_filepaths_midi = QtWidgets.QWidget()
        self.tw_filepaths_midi.setObjectName("tw_filepaths_midi")
        self.verticalLayout_23 = QtWidgets.QVBoxLayout(self.tw_filepaths_midi)
        self.verticalLayout_23.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_23.setSpacing(0)
        self.verticalLayout_23.setObjectName("verticalLayout_23")
        self.lw_files_midi = QtWidgets.QListWidget(self.tw_filepaths_midi)
        self.lw_files_midi.setObjectName("lw_files_midi")
        self.verticalLayout_23.addWidget(self.lw_files_midi)
        self.tw_filepaths.addWidget(self.tw_filepaths_midi)
        self.horizontalLayout_16.addWidget(self.tw_filepaths)
        self.verticalLayout_29 = QtWidgets.QVBoxLayout()
        self.verticalLayout_29.setObjectName("verticalLayout_29")
        self.b_filepaths_add = QtWidgets.QPushButton(self.page_filepaths)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/16x16/list-add.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_filepaths_add.setIcon(icon8)
        self.b_filepaths_add.setObjectName("b_filepaths_add")
        self.verticalLayout_29.addWidget(self.b_filepaths_add)
        self.b_filepaths_remove = QtWidgets.QPushButton(self.page_filepaths)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/16x16/list-remove.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_filepaths_remove.setIcon(icon9)
        self.b_filepaths_remove.setObjectName("b_filepaths_remove")
        self.verticalLayout_29.addWidget(self.b_filepaths_remove)
        spacerItem36 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_29.addItem(spacerItem36)
        self.b_filepaths_change = QtWidgets.QPushButton(self.page_filepaths)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/16x16/edit-rename.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_filepaths_change.setIcon(icon10)
        self.b_filepaths_change.setObjectName("b_filepaths_change")
        self.verticalLayout_29.addWidget(self.b_filepaths_change)
        spacerItem37 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_29.addItem(spacerItem37)
        self.horizontalLayout_16.addLayout(self.verticalLayout_29)
        self.verticalLayout_30.addLayout(self.horizontalLayout_16)
        self.stackedWidget.addWidget(self.page_filepaths)
        self.page_pluginpaths = QtWidgets.QWidget()
        self.page_pluginpaths.setObjectName("page_pluginpaths")
        self.verticalLayout_19 = QtWidgets.QVBoxLayout(self.page_pluginpaths)
        self.verticalLayout_19.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_19.setObjectName("verticalLayout_19")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_18 = QtWidgets.QLabel(self.page_pluginpaths)
        self.label_18.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_6.addWidget(self.label_18)
        spacerItem38 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem38)
        self.label_19 = QtWidgets.QLabel(self.page_pluginpaths)
        self.label_19.setMaximumSize(QtCore.QSize(48, 48))
        self.label_19.setText("")
        self.label_19.setPixmap(QtGui.QPixmap(":/scalable/folder.svgz"))
        self.label_19.setScaledContents(True)
        self.label_19.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_19.setWordWrap(True)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_6.addWidget(self.label_19)
        self.verticalLayout_19.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.cb_paths = QtWidgets.QComboBox(self.page_pluginpaths)
        self.cb_paths.setMinimumSize(QtCore.QSize(120, 0))
        self.cb_paths.setObjectName("cb_paths")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.cb_paths.addItem("")
        self.horizontalLayout_9.addWidget(self.cb_paths)
        spacerItem39 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem39)
        self.verticalLayout_19.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.tw_paths = QtWidgets.QStackedWidget(self.page_pluginpaths)
        self.tw_paths.setObjectName("tw_paths")
        self.tw_paths_ladspa = QtWidgets.QWidget()
        self.tw_paths_ladspa.setObjectName("tw_paths_ladspa")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.tw_paths_ladspa)
        self.verticalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_11.setSpacing(0)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.lw_ladspa = QtWidgets.QListWidget(self.tw_paths_ladspa)
        self.lw_ladspa.setObjectName("lw_ladspa")
        self.verticalLayout_11.addWidget(self.lw_ladspa)
        self.tw_paths.addWidget(self.tw_paths_ladspa)
        self.tw_paths_dssi = QtWidgets.QWidget()
        self.tw_paths_dssi.setObjectName("tw_paths_dssi")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.tw_paths_dssi)
        self.verticalLayout_12.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_12.setSpacing(0)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.lw_dssi = QtWidgets.QListWidget(self.tw_paths_dssi)
        self.lw_dssi.setObjectName("lw_dssi")
        self.verticalLayout_12.addWidget(self.lw_dssi)
        self.tw_paths.addWidget(self.tw_paths_dssi)
        self.tw_paths_lv2 = QtWidgets.QWidget()
        self.tw_paths_lv2.setObjectName("tw_paths_lv2")
        self.gridLayout = QtWidgets.QGridLayout(self.tw_paths_lv2)
        self.gridLayout.setContentsMargins(0, 0, 0, 6)
        self.gridLayout.setObjectName("gridLayout")
        self.lw_lv2 = QtWidgets.QListWidget(self.tw_paths_lv2)
        self.lw_lv2.setObjectName("lw_lv2")
        self.gridLayout.addWidget(self.lw_lv2, 0, 0, 1, 2)
        self.label_4 = QtWidgets.QLabel(self.tw_paths_lv2)
        self.label_4.setMaximumSize(QtCore.QSize(22, 22))
        self.label_4.setText("")
        self.label_4.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_4.setScaledContents(True)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.tw_paths_lv2)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 1, 1, 1)
        self.tw_paths.addWidget(self.tw_paths_lv2)
        self.tw_paths_vst = QtWidgets.QWidget()
        self.tw_paths_vst.setObjectName("tw_paths_vst")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.tw_paths_vst)
        self.verticalLayout_13.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_13.setSpacing(0)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.lw_vst = QtWidgets.QListWidget(self.tw_paths_vst)
        self.lw_vst.setObjectName("lw_vst")
        self.verticalLayout_13.addWidget(self.lw_vst)
        self.tw_paths.addWidget(self.tw_paths_vst)
        self.tw_paths_vst3 = QtWidgets.QWidget()
        self.tw_paths_vst3.setObjectName("tw_paths_vst3")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.tw_paths_vst3)
        self.verticalLayout_15.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_15.setSpacing(0)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.lw_vst3 = QtWidgets.QListWidget(self.tw_paths_vst3)
        self.lw_vst3.setObjectName("lw_vst3")
        self.verticalLayout_15.addWidget(self.lw_vst3)
        self.tw_paths.addWidget(self.tw_paths_vst3)
        self.tw_paths_sf2 = QtWidgets.QWidget()
        self.tw_paths_sf2.setObjectName("tw_paths_sf2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tw_paths_sf2)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.lw_sf2 = QtWidgets.QListWidget(self.tw_paths_sf2)
        self.lw_sf2.setObjectName("lw_sf2")
        self.verticalLayout_4.addWidget(self.lw_sf2)
        self.tw_paths.addWidget(self.tw_paths_sf2)
        self.tw_paths_sfz = QtWidgets.QWidget()
        self.tw_paths_sfz.setObjectName("tw_paths_sfz")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.tw_paths_sfz)
        self.verticalLayout_14.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_14.setSpacing(0)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.lw_sfz = QtWidgets.QListWidget(self.tw_paths_sfz)
        self.lw_sfz.setObjectName("lw_sfz")
        self.verticalLayout_14.addWidget(self.lw_sfz)
        self.tw_paths.addWidget(self.tw_paths_sfz)
        self.tw_paths_jsfx = QtWidgets.QWidget()
        self.tw_paths_jsfx.setObjectName("tw_paths_jsfx")
        self.verticalLayout_24 = QtWidgets.QVBoxLayout(self.tw_paths_jsfx)
        self.verticalLayout_24.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_24.setSpacing(0)
        self.verticalLayout_24.setObjectName("verticalLayout_24")
        self.lw_jsfx = QtWidgets.QListWidget(self.tw_paths_jsfx)
        self.lw_jsfx.setObjectName("lw_jsfx")
        self.verticalLayout_24.addWidget(self.lw_jsfx)
        self.tw_paths.addWidget(self.tw_paths_jsfx)
        self.horizontalLayout_11.addWidget(self.tw_paths)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.b_paths_add = QtWidgets.QPushButton(self.page_pluginpaths)
        self.b_paths_add.setIcon(icon8)
        self.b_paths_add.setObjectName("b_paths_add")
        self.verticalLayout.addWidget(self.b_paths_add)
        self.b_paths_remove = QtWidgets.QPushButton(self.page_pluginpaths)
        self.b_paths_remove.setIcon(icon9)
        self.b_paths_remove.setObjectName("b_paths_remove")
        self.verticalLayout.addWidget(self.b_paths_remove)
        spacerItem40 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem40)
        self.b_paths_change = QtWidgets.QPushButton(self.page_pluginpaths)
        self.b_paths_change.setIcon(icon10)
        self.b_paths_change.setObjectName("b_paths_change")
        self.verticalLayout.addWidget(self.b_paths_change)
        spacerItem41 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem41)
        self.horizontalLayout_11.addLayout(self.verticalLayout)
        self.verticalLayout_19.addLayout(self.horizontalLayout_11)
        self.stackedWidget.addWidget(self.page_pluginpaths)
        self.page_wine = QtWidgets.QWidget()
        self.page_wine.setObjectName("page_wine")
        self.verticalLayout_25 = QtWidgets.QVBoxLayout(self.page_wine)
        self.verticalLayout_25.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_25.setObjectName("verticalLayout_25")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_29 = QtWidgets.QLabel(self.page_wine)
        self.label_29.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_29.setObjectName("label_29")
        self.horizontalLayout_17.addWidget(self.label_29)
        spacerItem42 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem42)
        self.label_icon_wine = QtWidgets.QLabel(self.page_wine)
        self.label_icon_wine.setMaximumSize(QtCore.QSize(48, 48))
        self.label_icon_wine.setText("")
        self.label_icon_wine.setPixmap(QtGui.QPixmap(":/scalable/wine.svgz"))
        self.label_icon_wine.setScaledContents(True)
        self.label_icon_wine.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_icon_wine.setObjectName("label_icon_wine")
        self.horizontalLayout_17.addWidget(self.label_icon_wine)
        self.verticalLayout_25.addLayout(self.horizontalLayout_17)
        self.group_wine_exec = QtWidgets.QGroupBox(self.page_wine)
        self.group_wine_exec.setObjectName("group_wine_exec")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout(self.group_wine_exec)
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_8 = QtWidgets.QLabel(self.group_wine_exec)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_18.addWidget(self.label_8)
        self.le_wine_exec = QtWidgets.QLineEdit(self.group_wine_exec)
        self.le_wine_exec.setObjectName("le_wine_exec")
        self.horizontalLayout_18.addWidget(self.le_wine_exec)
        self.verticalLayout_25.addWidget(self.group_wine_exec)
        self.group_wine_prefix = QtWidgets.QGroupBox(self.page_wine)
        self.group_wine_prefix.setObjectName("group_wine_prefix")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.group_wine_prefix)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.cb_wine_prefix_detect = QtWidgets.QCheckBox(self.group_wine_prefix)
        self.cb_wine_prefix_detect.setObjectName("cb_wine_prefix_detect")
        self.gridLayout_6.addWidget(self.cb_wine_prefix_detect, 0, 0, 1, 2)
        self.label = QtWidgets.QLabel(self.group_wine_prefix)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridLayout_6.addWidget(self.label, 1, 0, 1, 1)
        self.le_wine_prefix_fallback = QtWidgets.QLineEdit(self.group_wine_prefix)
        self.le_wine_prefix_fallback.setObjectName("le_wine_prefix_fallback")
        self.gridLayout_6.addWidget(self.le_wine_prefix_fallback, 1, 1, 1, 1)
        self.label_30 = QtWidgets.QLabel(self.group_wine_prefix)
        self.label_30.setAlignment(QtCore.Qt.AlignCenter)
        self.label_30.setObjectName("label_30")
        self.gridLayout_6.addWidget(self.label_30, 2, 0, 1, 2)
        self.verticalLayout_25.addWidget(self.group_wine_prefix)
        self.group_wine_realtime = QtWidgets.QGroupBox(self.page_wine)
        self.group_wine_realtime.setCheckable(True)
        self.group_wine_realtime.setObjectName("group_wine_realtime")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.group_wine_realtime)
        self.gridLayout_5.setObjectName("gridLayout_5")
        spacerItem43 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_5.addItem(spacerItem43, 1, 3, 1, 1)
        spacerItem44 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_5.addItem(spacerItem44, 0, 3, 1, 1)
        self.sb_wine_base_prio = QtWidgets.QSpinBox(self.group_wine_realtime)
        self.sb_wine_base_prio.setMinimum(1)
        self.sb_wine_base_prio.setMaximum(89)
        self.sb_wine_base_prio.setObjectName("sb_wine_base_prio")
        self.gridLayout_5.addWidget(self.sb_wine_base_prio, 0, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.group_wine_realtime)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_5.addWidget(self.label_2, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.group_wine_realtime)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_5.addWidget(self.label_3, 1, 1, 1, 1)
        self.sb_wine_server_prio = QtWidgets.QSpinBox(self.group_wine_realtime)
        self.sb_wine_server_prio.setMinimum(1)
        self.sb_wine_server_prio.setMaximum(99)
        self.sb_wine_server_prio.setObjectName("sb_wine_server_prio")
        self.gridLayout_5.addWidget(self.sb_wine_server_prio, 1, 2, 1, 1)
        spacerItem45 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_5.addItem(spacerItem45, 0, 0, 1, 1)
        spacerItem46 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_5.addItem(spacerItem46, 1, 0, 1, 1)
        self.verticalLayout_25.addWidget(self.group_wine_realtime)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        spacerItem47 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem47)
        self.label_26 = QtWidgets.QLabel(self.page_wine)
        self.label_26.setMaximumSize(QtCore.QSize(22, 22))
        self.label_26.setText("")
        self.label_26.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_26.setScaledContents(True)
        self.label_26.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_14.addWidget(self.label_26)
        self.label_27 = QtWidgets.QLabel(self.page_wine)
        self.label_27.setObjectName("label_27")
        self.horizontalLayout_14.addWidget(self.label_27)
        spacerItem48 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem48)
        self.verticalLayout_25.addLayout(self.horizontalLayout_14)
        spacerItem49 = QtWidgets.QSpacerItem(20, 96, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_25.addItem(spacerItem49)
        self.label_28 = QtWidgets.QLabel(self.page_wine)
        self.label_28.setMaximumSize(QtCore.QSize(16777215, 5))
        self.label_28.setText("")
        self.label_28.setObjectName("label_28")
        self.verticalLayout_25.addWidget(self.label_28)
        self.stackedWidget.addWidget(self.page_wine)
        self.page_experimental = QtWidgets.QWidget()
        self.page_experimental.setObjectName("page_experimental")
        self.verticalLayout_21 = QtWidgets.QVBoxLayout(self.page_experimental)
        self.verticalLayout_21.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_21.setObjectName("verticalLayout_21")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label_11 = QtWidgets.QLabel(self.page_experimental)
        self.label_11.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_15.addWidget(self.label_11)
        spacerItem50 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem50)
        self.label_icon_experimental = QtWidgets.QLabel(self.page_experimental)
        self.label_icon_experimental.setMaximumSize(QtCore.QSize(48, 48))
        self.label_icon_experimental.setText("")
        self.label_icon_experimental.setPixmap(QtGui.QPixmap(":/scalable/warning.svgz"))
        self.label_icon_experimental.setScaledContents(True)
        self.label_icon_experimental.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label_icon_experimental.setObjectName("label_icon_experimental")
        self.horizontalLayout_15.addWidget(self.label_icon_experimental)
        self.verticalLayout_21.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        spacerItem51 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem51)
        self.label_16 = QtWidgets.QLabel(self.page_experimental)
        self.label_16.setMaximumSize(QtCore.QSize(22, 22))
        self.label_16.setText("")
        self.label_16.setPixmap(QtGui.QPixmap(":/16x16/dialog-warning.svgz"))
        self.label_16.setScaledContents(True)
        self.label_16.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_16.setWordWrap(True)
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_12.addWidget(self.label_16)
        self.label_17 = QtWidgets.QLabel(self.page_experimental)
        self.label_17.setObjectName("label_17")
        self.horizontalLayout_12.addWidget(self.label_17)
        spacerItem52 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem52)
        self.verticalLayout_21.addLayout(self.horizontalLayout_12)
        self.group_experimental_main = QtWidgets.QGroupBox(self.page_experimental)
        self.group_experimental_main.setObjectName("group_experimental_main")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(self.group_experimental_main)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.cb_exp_plugin_bridges = QtWidgets.QCheckBox(self.group_experimental_main)
        self.cb_exp_plugin_bridges.setObjectName("cb_exp_plugin_bridges")
        self.verticalLayout_17.addWidget(self.cb_exp_plugin_bridges)
        self.ch_exp_wine_bridges = QtWidgets.QCheckBox(self.group_experimental_main)
        self.ch_exp_wine_bridges.setEnabled(False)
        self.ch_exp_wine_bridges.setObjectName("ch_exp_wine_bridges")
        self.verticalLayout_17.addWidget(self.ch_exp_wine_bridges)
        self.ch_exp_jack_apps = QtWidgets.QCheckBox(self.group_experimental_main)
        self.ch_exp_jack_apps.setObjectName("ch_exp_jack_apps")
        self.verticalLayout_17.addWidget(self.ch_exp_jack_apps)
        self.ch_exp_export_lv2 = QtWidgets.QCheckBox(self.group_experimental_main)
        self.ch_exp_export_lv2.setObjectName("ch_exp_export_lv2")
        self.verticalLayout_17.addWidget(self.ch_exp_export_lv2)
        self.ch_main_system_icons = QtWidgets.QCheckBox(self.group_experimental_main)
        self.ch_main_system_icons.setObjectName("ch_main_system_icons")
        self.verticalLayout_17.addWidget(self.ch_main_system_icons)
        self.ch_exp_load_lib_global = QtWidgets.QCheckBox(self.group_experimental_main)
        self.ch_exp_load_lib_global.setObjectName("ch_exp_load_lib_global")
        self.verticalLayout_17.addWidget(self.ch_exp_load_lib_global)
        self.verticalLayout_21.addWidget(self.group_experimental_main)
        self.group_experimental_canvas = QtWidgets.QGroupBox(self.page_experimental)
        self.group_experimental_canvas.setObjectName("group_experimental_canvas")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout(self.group_experimental_canvas)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.cb_canvas_fancy_eyecandy = QtWidgets.QCheckBox(self.group_experimental_canvas)
        self.cb_canvas_fancy_eyecandy.setObjectName("cb_canvas_fancy_eyecandy")
        self.verticalLayout_16.addWidget(self.cb_canvas_fancy_eyecandy)
        self.cb_canvas_use_opengl = QtWidgets.QCheckBox(self.group_experimental_canvas)
        self.cb_canvas_use_opengl.setObjectName("cb_canvas_use_opengl")
        self.verticalLayout_16.addWidget(self.cb_canvas_use_opengl)
        self.cb_canvas_render_hq_aa = QtWidgets.QCheckBox(self.group_experimental_canvas)
        self.cb_canvas_render_hq_aa.setEnabled(False)
        self.cb_canvas_render_hq_aa.setObjectName("cb_canvas_render_hq_aa")
        self.verticalLayout_16.addWidget(self.cb_canvas_render_hq_aa)
        self.cb_canvas_inline_displays = QtWidgets.QCheckBox(self.group_experimental_canvas)
        self.cb_canvas_inline_displays.setObjectName("cb_canvas_inline_displays")
        self.verticalLayout_16.addWidget(self.cb_canvas_inline_displays)
        self.verticalLayout_21.addWidget(self.group_experimental_canvas)
        self.group_experimental_engine = QtWidgets.QGroupBox(self.page_experimental)
        self.group_experimental_engine.setObjectName("group_experimental_engine")
        self.verticalLayout_20 = QtWidgets.QVBoxLayout(self.group_experimental_engine)
        self.verticalLayout_20.setObjectName("verticalLayout_20")
        self.ch_engine_force_stereo = QtWidgets.QCheckBox(self.group_experimental_engine)
        self.ch_engine_force_stereo.setObjectName("ch_engine_force_stereo")
        self.verticalLayout_20.addWidget(self.ch_engine_force_stereo)
        self.ch_exp_prevent_bad_behaviour = QtWidgets.QCheckBox(self.group_experimental_engine)
        self.ch_exp_prevent_bad_behaviour.setObjectName("ch_exp_prevent_bad_behaviour")
        self.verticalLayout_20.addWidget(self.ch_exp_prevent_bad_behaviour)
        self.ch_engine_prefer_plugin_bridges = QtWidgets.QCheckBox(self.group_experimental_engine)
        self.ch_engine_prefer_plugin_bridges.setEnabled(False)
        self.ch_engine_prefer_plugin_bridges.setObjectName("ch_engine_prefer_plugin_bridges")
        self.verticalLayout_20.addWidget(self.ch_engine_prefer_plugin_bridges)
        self.verticalLayout_21.addWidget(self.group_experimental_engine)
        spacerItem53 = QtWidgets.QSpacerItem(20, 70, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_21.addItem(spacerItem53)
        self.label_25 = QtWidgets.QLabel(self.page_experimental)
        self.label_25.setMaximumSize(QtCore.QSize(16777215, 5))
        self.label_25.setText("")
        self.label_25.setObjectName("label_25")
        self.verticalLayout_21.addWidget(self.label_25)
        self.stackedWidget.addWidget(self.page_experimental)
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.stackedWidget.addWidget(self.page)
        self.horizontalLayout_3.addWidget(self.stackedWidget)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.buttonBox = QtWidgets.QDialogButtonBox(CarlaSettingsW)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok|QtWidgets.QDialogButtonBox.Reset)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(CarlaSettingsW)
        self.stackedWidget.setCurrentIndex(0)
        self.sw_engine_process_mode.setCurrentIndex(1)
        self.cb_engine_process_mode_jack.setCurrentIndex(0)
        self.tw_filepaths_info.setCurrentIndex(0)
        self.tw_filepaths.setCurrentIndex(0)
        self.tw_paths.setCurrentIndex(0)
        self.buttonBox.accepted.connect(CarlaSettingsW.accept) # type: ignore
        self.buttonBox.rejected.connect(CarlaSettingsW.reject) # type: ignore
        self.lw_page.currentCellChanged['int','int','int','int'].connect(self.stackedWidget.setCurrentIndex) # type: ignore
        self.ch_main_theme_pro.toggled['bool'].connect(self.label_main_theme_color.setEnabled) # type: ignore
        self.ch_main_theme_pro.toggled['bool'].connect(self.cb_main_theme_color.setEnabled) # type: ignore
        self.cb_paths.currentIndexChanged['int'].connect(self.tw_paths.setCurrentIndex) # type: ignore
        self.cb_canvas_use_opengl.toggled['bool'].connect(self.cb_canvas_render_hq_aa.setEnabled) # type: ignore
        self.cb_exp_plugin_bridges.toggled['bool'].connect(self.ch_exp_wine_bridges.setEnabled) # type: ignore
        self.cb_exp_plugin_bridges.toggled['bool'].connect(self.ch_engine_prefer_plugin_bridges.setEnabled) # type: ignore
        self.rb_osc_tcp_port_specific.toggled['bool'].connect(self.sb_osc_tcp_port_number.setEnabled) # type: ignore
        self.rb_osc_udp_port_specific.toggled['bool'].connect(self.sb_osc_udp_port_number.setEnabled) # type: ignore
        self.cb_filepaths.currentIndexChanged['int'].connect(self.tw_filepaths.setCurrentIndex) # type: ignore
        self.cb_filepaths.currentIndexChanged['int'].connect(self.tw_filepaths_info.setCurrentIndex) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(CarlaSettingsW)

    def retranslateUi(self, CarlaSettingsW):
        _translate = QtCore.QCoreApplication.translate
        CarlaSettingsW.setWindowTitle(_translate("CarlaSettingsW", "Settings"))
        item = self.lw_page.verticalHeaderItem(0)
        item.setText(_translate("CarlaSettingsW", "main"))
        item = self.lw_page.verticalHeaderItem(1)
        item.setText(_translate("CarlaSettingsW", "canvas"))
        item = self.lw_page.verticalHeaderItem(2)
        item.setText(_translate("CarlaSettingsW", "engine"))
        item = self.lw_page.verticalHeaderItem(3)
        item.setText(_translate("CarlaSettingsW", "osc"))
        item = self.lw_page.verticalHeaderItem(4)
        item.setText(_translate("CarlaSettingsW", "file-paths"))
        item = self.lw_page.verticalHeaderItem(5)
        item.setText(_translate("CarlaSettingsW", "plugin-paths"))
        item = self.lw_page.verticalHeaderItem(6)
        item.setText(_translate("CarlaSettingsW", "wine"))
        item = self.lw_page.verticalHeaderItem(7)
        item.setText(_translate("CarlaSettingsW", "experimental"))
        item = self.lw_page.horizontalHeaderItem(0)
        item.setText(_translate("CarlaSettingsW", "Widget"))
        __sortingEnabled = self.lw_page.isSortingEnabled()
        self.lw_page.setSortingEnabled(False)
        item = self.lw_page.item(0, 0)
        item.setText(_translate("CarlaSettingsW", "Main"))
        item = self.lw_page.item(1, 0)
        item.setText(_translate("CarlaSettingsW", "Canvas"))
        item = self.lw_page.item(2, 0)
        item.setText(_translate("CarlaSettingsW", "Engine"))
        item = self.lw_page.item(4, 0)
        item.setText(_translate("CarlaSettingsW", "File Paths"))
        item = self.lw_page.item(5, 0)
        item.setText(_translate("CarlaSettingsW", "Plugin Paths"))
        item = self.lw_page.item(6, 0)
        item.setText(_translate("CarlaSettingsW", "Wine"))
        item = self.lw_page.item(7, 0)
        item.setText(_translate("CarlaSettingsW", "Experimental"))
        self.lw_page.setSortingEnabled(__sortingEnabled)
        self.label_main.setText(_translate("CarlaSettingsW", "<b>Main</b>"))
        self.group_main_paths.setTitle(_translate("CarlaSettingsW", "Paths"))
        self.label_main_proj_folder_open.setText(_translate("CarlaSettingsW", "Default project folder:"))
        self.group_main_misc.setTitle(_translate("CarlaSettingsW", "Interface"))
        self.cb_main_classic_skin_default.setText(_translate("CarlaSettingsW", "Use \"Classic\" as default rack skin"))
        self.label_main_refresh_interval.setText(_translate("CarlaSettingsW", "Interface refresh interval:"))
        self.sb_main_refresh_interval.setSuffix(_translate("CarlaSettingsW", " ms"))
        self.ch_main_show_logs.setText(_translate("CarlaSettingsW", "Show console output in Logs tab (needs engine restart)"))
        self.ch_main_confirm_exit.setText(_translate("CarlaSettingsW", "Show a confirmation dialog before quitting"))
        self.group_main_theme.setTitle(_translate("CarlaSettingsW", "Theme"))
        self.ch_main_theme_pro.setText(_translate("CarlaSettingsW", "Use Carla \"PRO\" theme (needs restart)"))
        self.label_main_theme_color.setText(_translate("CarlaSettingsW", "Color scheme:"))
        self.cb_main_theme_color.setItemText(0, _translate("CarlaSettingsW", "Black"))
        self.cb_main_theme_color.setItemText(1, _translate("CarlaSettingsW", "System"))
        self.group_main_experimental.setTitle(_translate("CarlaSettingsW", "Experimental"))
        self.ch_main_experimental.setText(_translate("CarlaSettingsW", "Enable experimental features"))
        self.label_9.setText(_translate("CarlaSettingsW", "<b>Canvas</b>"))
        self.group_canvas_theme.setTitle(_translate("CarlaSettingsW", "Theme"))
        self.cb_canvas_bezier_lines.setText(_translate("CarlaSettingsW", "Bezier Lines"))
        self.label_canvas_theme.setText(_translate("CarlaSettingsW", "Theme:"))
        self.label_canvas_size.setText(_translate("CarlaSettingsW", "Size:"))
        self.cb_canvas_size.setItemText(0, _translate("CarlaSettingsW", "775x600"))
        self.cb_canvas_size.setItemText(1, _translate("CarlaSettingsW", "1550x1200"))
        self.cb_canvas_size.setItemText(2, _translate("CarlaSettingsW", "3100x2400"))
        self.cb_canvas_size.setItemText(3, _translate("CarlaSettingsW", "4650x3600"))
        self.cb_canvas_size.setItemText(4, _translate("CarlaSettingsW", "6200x4800"))
        self.cb_canvas_size.setItemText(5, _translate("CarlaSettingsW", "12400x9600"))
        self.group_canvas_options.setTitle(_translate("CarlaSettingsW", "Options"))
        self.cb_canvas_hide_groups.setText(_translate("CarlaSettingsW", "Auto-hide groups with no ports"))
        self.cb_canvas_auto_select.setText(_translate("CarlaSettingsW", "Auto-select items on hover"))
        self.cb_canvas_eyecandy.setText(_translate("CarlaSettingsW", "Basic eye-candy (group shadows)"))
        self.group_canvas_render.setTitle(_translate("CarlaSettingsW", "Render Hints"))
        self.cb_canvas_render_aa.setText(_translate("CarlaSettingsW", "Anti-Aliasing"))
        self.cb_canvas_full_repaints.setText(_translate("CarlaSettingsW", "Full canvas repaints (slower, but prevents drawing issues)"))
        self.label_6.setText(_translate("CarlaSettingsW", "<b>Engine</b>"))
        self.groupBox.setTitle(_translate("CarlaSettingsW", "Core"))
        self.cb_engine_process_mode_jack.setItemText(0, _translate("CarlaSettingsW", "Single Client"))
        self.cb_engine_process_mode_jack.setItemText(1, _translate("CarlaSettingsW", "Multiple Clients"))
        self.cb_engine_process_mode_jack.setItemText(2, _translate("CarlaSettingsW", "Continuous Rack"))
        self.cb_engine_process_mode_jack.setItemText(3, _translate("CarlaSettingsW", "Patchbay"))
        self.cb_engine_process_mode_other.setItemText(0, _translate("CarlaSettingsW", "Continuous Rack"))
        self.cb_engine_process_mode_other.setItemText(1, _translate("CarlaSettingsW", "Patchbay"))
        self.label_21.setText(_translate("CarlaSettingsW", "Audio driver:"))
        self.label_20.setText(_translate("CarlaSettingsW", "Process mode:"))
        self.label_7.setToolTip(_translate("CarlaSettingsW", "Maximum number of parameters to allow in the built-in \'Edit\' dialog"))
        self.label_7.setText(_translate("CarlaSettingsW", "Max Parameters:"))
        self.sb_engine_max_params.setToolTip(_translate("CarlaSettingsW", "Maximum number of parameters to allow in the built-in \'Edit\' dialog"))
        self.tb_engine_driver_config.setText(_translate("CarlaSettingsW", "..."))
        self.cb_engine_reset_xruns.setText(_translate("CarlaSettingsW", "Reset Xrun counter after project load"))
        self.groupBox_8.setTitle(_translate("CarlaSettingsW", "Plugin UIs"))
        self.label_engine_ui_bridges_timeout.setToolTip(_translate("CarlaSettingsW", "How much time to wait for OSC GUIs to ping back the host"))
        self.label_engine_ui_bridges_timeout.setText(_translate("CarlaSettingsW", "UI Bridge Timeout:"))
        self.sb_engine_ui_bridges_timeout.setToolTip(_translate("CarlaSettingsW", "How much time to wait for OSC GUIs to ping back the host"))
        self.sb_engine_ui_bridges_timeout.setSuffix(_translate("CarlaSettingsW", " ms"))
        self.ch_engine_prefer_ui_bridges.setToolTip(_translate("CarlaSettingsW", "Use OSC-GUI bridges when possible, this way separating the UI from DSP code"))
        self.ch_engine_prefer_ui_bridges.setText(_translate("CarlaSettingsW", "Use UI bridges instead of direct handling when possible"))
        self.ch_engine_uis_always_on_top.setText(_translate("CarlaSettingsW", "Make plugin UIs always-on-top"))
        self.ch_engine_manage_uis.setText(_translate("CarlaSettingsW", "Make plugin UIs appear on top of Carla (needs restart)"))
        self.label_engine_ui_bridges_mac_note.setText(_translate("CarlaSettingsW", "NOTE: Plugin-bridge UIs cannot be managed by Carla on macOS"))
        self.label_23.setText(_translate("CarlaSettingsW", "Restart the engine to load the new settings"))
        self.label_14.setText(_translate("CarlaSettingsW", "<b>OSC</b>"))
        self.group_osc_core.setTitle(_translate("CarlaSettingsW", "Core"))
        self.ch_osc_enable.setText(_translate("CarlaSettingsW", "Enable OSC"))
        self.group_osc_tcp_port.setTitle(_translate("CarlaSettingsW", "Enable TCP port"))
        self.rb_osc_tcp_port_specific.setText(_translate("CarlaSettingsW", "Use specific port:"))
        self.label_40.setText(_translate("CarlaSettingsW", "Overridden by CARLA_OSC_TCP_PORT env var"))
        self.rb_osc_tcp_port_random.setText(_translate("CarlaSettingsW", "Use randomly assigned port"))
        self.group_osc_udp_port.setTitle(_translate("CarlaSettingsW", "Enable UDP port"))
        self.rb_osc_udp_port_specific.setText(_translate("CarlaSettingsW", "Use specific port:"))
        self.rb_osc_udp_port_random.setText(_translate("CarlaSettingsW", "Use randomly assigned port"))
        self.label_42.setText(_translate("CarlaSettingsW", "Overridden by CARLA_OSC_UDP_PORT env var"))
        self.label_35.setText(_translate("CarlaSettingsW", "Restart the engine to load the new settings"))
        self.label_38.setText(_translate("CarlaSettingsW", "DSSI UIs require OSC UDP port enabled"))
        self.label_24.setText(_translate("CarlaSettingsW", "<b>File Paths</b>"))
        self.cb_filepaths.setItemText(0, _translate("CarlaSettingsW", "Audio"))
        self.cb_filepaths.setItemText(1, _translate("CarlaSettingsW", "MIDI"))
        self.label_12.setText(_translate("CarlaSettingsW", "Used for the \"audiofile\" plugin"))
        self.label_13.setText(_translate("CarlaSettingsW", "Used for the \"midifile\" plugin"))
        self.b_filepaths_add.setText(_translate("CarlaSettingsW", "Add..."))
        self.b_filepaths_remove.setText(_translate("CarlaSettingsW", "Remove"))
        self.b_filepaths_change.setText(_translate("CarlaSettingsW", "Change..."))
        self.label_18.setText(_translate("CarlaSettingsW", "<b>Plugin Paths</b>"))
        self.cb_paths.setItemText(0, _translate("CarlaSettingsW", "LADSPA"))
        self.cb_paths.setItemText(1, _translate("CarlaSettingsW", "DSSI"))
        self.cb_paths.setItemText(2, _translate("CarlaSettingsW", "LV2"))
        self.cb_paths.setItemText(3, _translate("CarlaSettingsW", "VST2"))
        self.cb_paths.setItemText(4, _translate("CarlaSettingsW", "VST3"))
        self.cb_paths.setItemText(5, _translate("CarlaSettingsW", "SF2/3"))
        self.cb_paths.setItemText(6, _translate("CarlaSettingsW", "SFZ"))
        self.cb_paths.setItemText(7, _translate("CarlaSettingsW", "JSFX"))
        self.label_5.setText(_translate("CarlaSettingsW", "Restart Carla to find new plugins"))
        self.b_paths_add.setText(_translate("CarlaSettingsW", "Add..."))
        self.b_paths_remove.setText(_translate("CarlaSettingsW", "Remove"))
        self.b_paths_change.setText(_translate("CarlaSettingsW", "Change..."))
        self.label_29.setText(_translate("CarlaSettingsW", "<b>Wine</b>"))
        self.group_wine_exec.setTitle(_translate("CarlaSettingsW", "Executable"))
        self.label_8.setText(_translate("CarlaSettingsW", "Path to \'wine\' binary:"))
        self.group_wine_prefix.setTitle(_translate("CarlaSettingsW", "Prefix"))
        self.cb_wine_prefix_detect.setText(_translate("CarlaSettingsW", "Auto-detect Wine prefix based on plugin filename"))
        self.label.setText(_translate("CarlaSettingsW", "Fallback:"))
        self.label_30.setText(_translate("CarlaSettingsW", "Note: WINEPREFIX env var is preferred over this fallback"))
        self.group_wine_realtime.setTitle(_translate("CarlaSettingsW", "Realtime Priority"))
        self.label_2.setText(_translate("CarlaSettingsW", "Base priority:"))
        self.label_3.setText(_translate("CarlaSettingsW", "WineServer priority:"))
        self.label_27.setText(_translate("CarlaSettingsW", "These options are not available for Carla as plugin"))
        self.label_11.setText(_translate("CarlaSettingsW", "<b>Experimental</b>"))
        self.label_17.setText(_translate("CarlaSettingsW", "Experimental options! Likely to be unstable!"))
        self.group_experimental_main.setTitle(_translate("CarlaSettingsW", "Main"))
        self.cb_exp_plugin_bridges.setText(_translate("CarlaSettingsW", "Enable plugin bridges"))
        self.ch_exp_wine_bridges.setText(_translate("CarlaSettingsW", "Enable Wine bridges"))
        self.ch_exp_jack_apps.setText(_translate("CarlaSettingsW", "Enable jack applications"))
        self.ch_exp_export_lv2.setText(_translate("CarlaSettingsW", "Export single plugins to LV2"))
        self.ch_main_system_icons.setText(_translate("CarlaSettingsW", "Use system/desktop-theme icons (needs restart)"))
        self.ch_exp_load_lib_global.setText(_translate("CarlaSettingsW", "Load Carla backend in global namespace (NOT RECOMMENDED)"))
        self.group_experimental_canvas.setTitle(_translate("CarlaSettingsW", "Canvas"))
        self.cb_canvas_fancy_eyecandy.setText(_translate("CarlaSettingsW", "Fancy eye-candy (fade-in/out groups, glow connections)"))
        self.cb_canvas_use_opengl.setText(_translate("CarlaSettingsW", "Use OpenGL for rendering (needs restart)"))
        self.cb_canvas_render_hq_aa.setText(_translate("CarlaSettingsW", "High Quality Anti-Aliasing (OpenGL only)"))
        self.cb_canvas_inline_displays.setText(_translate("CarlaSettingsW", "Render Ardour-style \"Inline Displays\""))
        self.group_experimental_engine.setTitle(_translate("CarlaSettingsW", "Engine"))
        self.ch_engine_force_stereo.setToolTip(_translate("CarlaSettingsW", "Force mono plugins as stereo by running 2 instances at the same time.\n"
"This mode is not available for VST plugins."))
        self.ch_engine_force_stereo.setText(_translate("CarlaSettingsW", "Force mono plugins as stereo"))
        self.ch_exp_prevent_bad_behaviour.setToolTip(_translate("CarlaSettingsW", "When enabled, Carla will try to prevent plugins from doing things that might mess up the audio or cause xruns, such as fork(), gtk_init() and similar."))
        self.ch_exp_prevent_bad_behaviour.setText(_translate("CarlaSettingsW", "Prevent unsafe calls from plugins (needs restart)"))
        self.ch_engine_prefer_plugin_bridges.setToolTip(_translate("CarlaSettingsW", "Run plugins in a separate process, so if they crash it does not affect Carla.\n"
"Plugins get automatically deactivated in such cases.\n"
"Reactvate them to start the process again, with the last saved state applied to it."))
        self.ch_engine_prefer_plugin_bridges.setText(_translate("CarlaSettingsW", "Run plugins in bridge mode when possible"))
import resources_rc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# PatchBay Canvas engine using QGraphicsView/Scene
# Copyright (C) 2010-2019 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import pyqtSlot, qCritical, qFatal, qWarning, QObject
from PyQt5.QtCore import QPointF, QRectF, QTimer
from PyQt5.QtWidgets import QGraphicsObject

# ------------------------------------------------------------------------------------------------------------
# Imports (Custom)

from . import (
    canvas,
    features,
    options,
    group_dict_t,
    port_dict_t,
    connection_dict_t,
    bool2str,
    icon2str,
    split2str,
    port_mode2str,
    port_type2str,
    CanvasIconType,
    CanvasRubberbandType,
    ACTION_PORTS_DISCONNECT,
    EYECANDY_FULL,
    ICON_APPLICATION,
    ICON_HARDWARE,
    ICON_LADISH_ROOM,
    PORT_MODE_INPUT,
    PORT_MODE_OUTPUT,
    SPLIT_YES,
    SPLIT_NO,
    SPLIT_UNDEF,
    MAX_PLUGIN_ID_ALLOWED,
)

from .canvasbox import CanvasBox
from .canvasbezierline import CanvasBezierLine
from .canvasline import CanvasLine
from .theme import Theme, getDefaultTheme, getThemeName
from .utils import CanvasCallback, CanvasGetNewGroupPos, CanvasItemFX, CanvasRemoveItemFX

# FIXME
from . import *
from .scene import PatchScene

from carla_shared import QSafeSettings

# ------------------------------------------------------------------------------------------------------------

class CanvasObject(QObject):
    def __init__(self, parent=None):
        QObject.__init__(self, parent)

    @pyqtSlot()
    def AnimationFinishedShow(self):
        animation = self.sender()
        if animation:
            animation.forceStop()
            canvas.animation_list.remove(animation)

    @pyqtSlot()
    def AnimationFinishedHide(self):
        animation = self.sender()
        if animation:
            animation.forceStop()
            canvas.animation_list.remove(animation)
            item = animation.item()
            if item:
                if isinstance(item, QGraphicsObject):
                    item.blockSignals(True)
                    item.hide()
                    item.blockSignals(False)
                else:
                    item.hide()

    @pyqtSlot()
    def AnimationFinishedDestroy(self):
        animation = self.sender()
        if animation:
            animation.forceStop()
            canvas.animation_list.remove(animation)
            item = animation.item()
            if item:
                CanvasRemoveItemFX(item)

    @pyqtSlot()
    def PortContextMenuConnect(self):
        try:
            sources, targets = self.sender().data()
        except:
            return

        for port_type in (PORT_TYPE_AUDIO_JACK, PORT_TYPE_MIDI_JACK, PORT_TYPE_MIDI_ALSA, PORT_TYPE_PARAMETER):
            source_ports = sources[port_type]
            target_ports = targets[port_type]

            source_ports_len = len(source_ports)
            target_ports_len = len(target_ports)

            if source_ports_len == 0 or target_ports_len == 0:
                continue

            for i in range(min(source_ports_len, target_ports_len)):
                data = "%i:%i:%i:%i" % (source_ports[i][0],
                                        source_ports[i][1],
                                        target_ports[i][0],
                                        target_ports[i][1])
                CanvasCallback(ACTION_PORTS_CONNECT, 0, 0, data)

            if source_ports_len == 1 and target_ports_len > 1:
                for i in range(1, target_ports_len):
                    data = "%i:%i:%i:%i" % (source_ports[0][0],
                                            source_ports[0][1],
                                            target_ports[i][0],
                                            target_ports[i][1])
                    CanvasCallback(ACTION_PORTS_CONNECT, 0, 0, data)

    @pyqtSlot()
    def PortContextMenuDisconnect(self):
        try:
            connectionId = int(self.sender().data())
        except:
            return

        CanvasCallback(ACTION_PORTS_DISCONNECT, connectionId, 0, "")

    @pyqtSlot(int, bool, int, int)
    def boxPositionChanged(self, groupId, split, x, y):
        x2 = y2 = 0

        if split:
            for group in canvas.group_list:
                if group.group_id == groupId:
                    if group.split:
                        pos = group.widgets[1].pos()
                        x2  = pos.x()
                        y2  = pos.y()
                    break

        valueStr = "%i:%i:%i:%i" % (x, y, x2, y2)
        CanvasCallback(ACTION_GROUP_POSITION, groupId, 0, valueStr)

    @pyqtSlot(int, bool, int, int)
    def sboxPositionChanged(self, groupId, split, x2, y2):
        x = y = 0

        for group in canvas.group_list:
            if group.group_id == groupId:
                pos = group.widgets[0].pos()
                x = pos.x()
                y = pos.y()
                break

        valueStr = "%i:%i:%i:%i" % (x, y, x2, y2)
        CanvasCallback(ACTION_GROUP_POSITION, groupId, 0, valueStr)

# ------------------------------------------------------------------------------------------------------------

def getStoredCanvasPosition(key, fallback_pos):
    try:
        return canvas.settings.value("CanvasPositions/" + key, fallback_pos, type=QPointF)
    except:
        return fallback_pos

def getStoredCanvasSplit(group_name, fallback_split_mode):
    try:
        return canvas.settings.value("CanvasPositions/%s_SPLIT" % group_name, fallback_split_mode, type=int)
    except:
        return fallback_split_mode

# ------------------------------------------------------------------------------------------------------------

def init(appName, scene, callback, debug=False):
    if debug:
        print("PatchCanvas::init(\"%s\", %s, %s, %s)" % (appName, scene, callback, bool2str(debug)))

    if canvas.initiated:
        qCritical("PatchCanvas::init() - already initiated")
        return

    if not callback:
        qFatal("PatchCanvas::init() - fatal error: callback not set")
        return

    canvas.callback = callback
    canvas.debug = debug
    canvas.scene = scene

    canvas.last_z_value = 0
    canvas.last_connection_id = 0
    canvas.initial_pos = QPointF(0, 0)
    canvas.size_rect = QRectF()

    if not canvas.qobject:
        canvas.qobject = CanvasObject()
    if not canvas.settings:
        canvas.settings = QSafeSettings("falkTX", appName)

    if canvas.theme:
        del canvas.theme
        canvas.theme = None

    for i in range(Theme.THEME_MAX):
        this_theme_name = getThemeName(i)
        if this_theme_name == options.theme_name:
            canvas.theme = Theme(i)
            break

    if not canvas.theme:
        canvas.theme = Theme(getDefaultTheme())

    canvas.scene.updateTheme()

    canvas.initiated = True

def clear():
    if canvas.debug:
        print("PatchCanvas::clear()")

    group_pos = {}
    group_list_ids = []
    port_list_ids = []
    connection_list_ids = []

    for group in canvas.group_list:
        group_pos[group.group_name] = (
            group.split,
            group.widgets[0].pos(),
            group.widgets[1].pos() if group.split else None,
        )
        group_list_ids.append(group.group_id)

    for port in canvas.port_list:
        port_list_ids.append((port.group_id, port.port_id))

    for connection in canvas.connection_list:
        connection_list_ids.append(connection.connection_id)

    for idx in connection_list_ids:
        disconnectPorts(idx)

    for group_id, port_id in port_list_ids:
        removePort(group_id, port_id)

    for idx in group_list_ids:
        removeGroup(idx)

    canvas.last_z_value = 0
    canvas.last_connection_id = 0

    canvas.group_list = []
    canvas.port_list = []
    canvas.connection_list = []
    canvas.group_plugin_map = {}
    canvas.old_group_pos = group_pos

    canvas.scene.clearSelection()

    animatedItems = []
    for animation in canvas.animation_list:
        animatedItems.append(animation.item())

    for item in canvas.scene.items():
        if item.type() in (CanvasIconType, CanvasRubberbandType) or item in animatedItems:
            continue
        canvas.scene.removeItem(item)
        del item

    canvas.initiated = False

    QTimer.singleShot(0, canvas.scene.update)

# ------------------------------------------------------------------------------------------------------------

def setInitialPos(x, y):
    if canvas.debug:
        print("PatchCanvas::setInitialPos(%i, %i)" % (x, y))

    canvas.initial_pos.setX(x)
    canvas.initial_pos.setY(y)

def setCanvasSize(x, y, width, height):
    if canvas.debug:
        print("PatchCanvas::setCanvasSize(%i, %i, %i, %i)" % (x, y, width, height))

    canvas.size_rect.setX(x)
    canvas.size_rect.setY(y)
    canvas.size_rect.setWidth(width)
    canvas.size_rect.setHeight(height)
    canvas.scene.updateLimits()
    canvas.scene.fixScaleFactor()

def addGroup(group_id, group_name, split=SPLIT_UNDEF, icon=ICON_APPLICATION):
    if canvas.debug:
        print("PatchCanvas::addGroup(%i, %s, %s, %s)" % (
              group_id, group_name.encode(), split2str(split), icon2str(icon)))

    for group in canvas.group_list:
        if group.group_id == group_id:
            qWarning("PatchCanvas::addGroup(%i, %s, %s, %s) - group already exists" % (
                     group_id, group_name.encode(), split2str(split), icon2str(icon)))
            return None

    old_matching_group = canvas.old_group_pos.pop(group_name, None)

    if split == SPLIT_UNDEF:
        isHardware = bool(icon == ICON_HARDWARE)

        if features.handle_group_pos:
            split = getStoredCanvasSplit(group_name, SPLIT_YES if isHardware else split)
        elif isHardware:
            split = SPLIT_YES
        elif old_matching_group is not None and old_matching_group[0]:
            split = SPLIT_YES

    group_box = CanvasBox(group_id, group_name, icon)
    group_box.positionChanged.connect(canvas.qobject.boxPositionChanged)
    group_box.blockSignals(True)

    group_dict = group_dict_t()
    group_dict.group_id = group_id
    group_dict.group_name = group_name
    group_dict.split = bool(split == SPLIT_YES)
    group_dict.icon = icon
    group_dict.plugin_id = -1
    group_dict.plugin_ui = False
    group_dict.plugin_inline = False
    group_dict.widgets = [group_box, None]

    if split == SPLIT_YES:
        group_box.setSplit(True, PORT_MODE_OUTPUT)

        if features.handle_group_pos:
            group_box.setPos(getStoredCanvasPosition(group_name + "_OUTPUT", CanvasGetNewGroupPos(False)))
        elif old_matching_group is not None:
            group_box.setPos(old_matching_group[1])
        else:
            group_box.setPos(CanvasGetNewGroupPos(False))

        group_sbox = CanvasBox(group_id, group_name, icon)
        group_sbox.positionChanged.connect(canvas.qobject.sboxPositionChanged)
        group_sbox.blockSignals(True)
        group_sbox.setSplit(True, PORT_MODE_INPUT)

        group_dict.widgets[1] = group_sbox

        if features.handle_group_pos:
            group_sbox.setPos(getStoredCanvasPosition(group_name + "_INPUT", CanvasGetNewGroupPos(True)))
        elif old_matching_group is not None and old_matching_group[0]:
            group_sbox.setPos(old_matching_group[2])
        else:
            group_sbox.setPos(group_box.x() + group_box.boundingRect().width() + 300, group_box.y())

        canvas.last_z_value += 1
        group_sbox.setZValue(canvas.last_z_value)

        if options.eyecandy == EYECANDY_FULL and not options.auto_hide_groups:
            CanvasItemFX(group_sbox, True, False)

        group_sbox.checkItemPos()
        group_sbox.blockSignals(False)

    else:
        group_box.setSplit(False)

        if features.handle_group_pos:
            group_box.setPos(getStoredCanvasPosition(group_name, CanvasGetNewGroupPos(False)))
        elif old_matching_group is not None:
            group_box.setPos(old_matching_group[1])
        else:
            # Special ladish fake-split groups
            horizontal = bool(icon == ICON_HARDWARE or icon == ICON_LADISH_ROOM)
            group_box.setPos(CanvasGetNewGroupPos(horizontal))

    canvas.last_z_value += 1
    group_box.setZValue(canvas.last_z_value)

    group_box.checkItemPos()
    group_box.blockSignals(False)

    canvas.group_list.append(group_dict)

    if options.eyecandy == EYECANDY_FULL and not options.auto_hide_groups:
        CanvasItemFX(group_box, True, False)
    else:
        QTimer.singleShot(0, canvas.scene.update)

    return group_dict

def removeGroup(group_id):
    if canvas.debug:
        print("PatchCanvas::removeGroup(%i)" % group_id)

    for group in canvas.group_list:
        if group.group_id == group_id:
            item = group.widgets[0]
            group_name = group.group_name

            if group.split:
                s_item = group.widgets[1]

                if features.handle_group_pos:
                    canvas.settings.setValue("CanvasPositions/%s_OUTPUT" % group_name, item.pos())
                    canvas.settings.setValue("CanvasPositions/%s_INPUT" % group_name, s_item.pos())
                    canvas.settings.setValue("CanvasPositions/%s_SPLIT" % group_name, SPLIT_YES)

                if options.eyecandy == EYECANDY_FULL:
                    CanvasItemFX(s_item, False, True)
                else:
                    s_item.removeIconFromScene()
                    canvas.scene.removeItem(s_item)
                    del s_item

            else:
                if features.handle_group_pos:
                    canvas.settings.setValue("CanvasPositions/%s" % group_name, item.pos())
                    canvas.settings.setValue("CanvasPositions/%s_SPLIT" % group_name, SPLIT_NO)

            if options.eyecandy == EYECANDY_FULL:
                CanvasItemFX(item, False, True)
            else:
                item.removeIconFromScene()
                canvas.scene.removeItem(item)
                del item

            canvas.group_list.remove(group)
            canvas.group_plugin_map.pop(group.plugin_id, None)

            QTimer.singleShot(0, canvas.scene.update)
            return

    qCritical("PatchCanvas::removeGroup(%i) - unable to find group to remove" % group_id)

def renameGroup(group_id, new_group_name):
    if canvas.debug:
        print("PatchCanvas::renameGroup(%i, %s)" % (group_id, new_group_name.encode()))

    for group in canvas.group_list:
        if group.group_id == group_id:
            group.group_name = new_group_name
            group.widgets[0].setGroupName(new_group_name)

            if group.split and group.widgets[1]:
                group.widgets[1].setGroupName(new_group_name)

            QTimer.singleShot(0, canvas.scene.update)
            return

    qCritical("PatchCanvas::renameGroup(%i, %s) - unable to find group to rename" % (group_id, new_group_name.encode()))

def splitGroup(group_id):
    if canvas.debug:
        print("PatchCanvas::splitGroup(%i)" % group_id)

    item = None
    group_name = ""
    group_icon = ICON_APPLICATION
    group_pos = None
    plugin_id = -1
    plugin_ui = False
    plugin_inline = False
    ports_data = []
    conns_data = []

    # Step 1 - Store all Item data
    for group in canvas.group_list:
        if group.group_id == group_id:
            if group.split:
                if canvas.debug:
                    print("PatchCanvas::splitGroup(%i) - group is already split" % group_id)
                return

            item = group.widgets[0]
            group_name = group.group_name
            group_icon = group.icon
            group_pos = item.pos()
            plugin_id = group.plugin_id
            plugin_ui = group.plugin_ui
            plugin_inline = group.plugin_inline
            break

    if not item:
        qCritical("PatchCanvas::splitGroup(%i) - unable to find group to split" % group_id)
        return

    port_list_ids = list(item.getPortList())

    for port in canvas.port_list:
        if port.group_id == group_id and port.port_id in port_list_ids:
            port_dict = port_dict_t()
            port_dict.group_id = port.group_id
            port_dict.port_id = port.port_id
            port_dict.port_name = port.port_name
            port_dict.port_mode = port.port_mode
            port_dict.port_type = port.port_type
            port_dict.is_alternate = port.is_alternate
            port_dict.widget = None
            ports_data.append(port_dict)

    for connection in canvas.connection_list:
        if (connection.group_in_id == group_id and connection.port_in_id in port_list_ids) or \
           (connection.group_out_id == group_id and connection.port_out_id in port_list_ids):
            connection_dict = connection_dict_t()
            connection_dict.connection_id = connection.connection_id
            connection_dict.group_in_id = connection.group_in_id
            connection_dict.port_in_id = connection.port_in_id
            connection_dict.group_out_id = connection.group_out_id
            connection_dict.port_out_id = connection.port_out_id
            connection_dict.widget = None
            conns_data.append(connection_dict)

    # Step 2 - Remove Item and Children
    for conn in conns_data:
        disconnectPorts(conn.connection_id)

    for port_id in port_list_ids:
        removePort(group_id, port_id)

    removeGroup(group_id)

    # Step 3 - Re-create Item, now split
    group = addGroup(group_id, group_name, SPLIT_YES, group_icon)

    if plugin_id >= 0:
        setGroupAsPlugin(group_id, plugin_id, plugin_ui, plugin_inline)

    for port in ports_data:
        addPort(group_id, port.port_id, port.port_name, port.port_mode, port.port_type, port.is_alternate)

    for conn in conns_data:
        connectPorts(conn.connection_id, conn.group_out_id, conn.port_out_id, conn.group_in_id, conn.port_in_id, True)

    if group is not None:
        pos1 = group.widgets[0].pos()
        pos2 = group.widgets[1].pos()
        group2_pos = QPointF(group_pos.x() + group.widgets[1].boundingRect().width() * 3/2, group_pos.y())
        group.widgets[0].blockSignals(True)
        group.widgets[0].setPos(group_pos)
        group.widgets[0].blockSignals(False)
        group.widgets[1].blockSignals(True)
        group.widgets[1].setPos(group2_pos)
        group.widgets[1].checkItemPos()
        group.widgets[1].blockSignals(False)
        valueStr = "%i:%i:%i:%i" % (group_pos.x(), group_pos.y(), group2_pos.x(), group2_pos.y())
        CanvasCallback(ACTION_GROUP_POSITION, group_id, 0, valueStr)

    QTimer.singleShot(0, canvas.scene.update)

def joinGroup(group_id):
    if canvas.debug:
        print("PatchCanvas::joinGroup(%i)" % group_id)

    item = None
    s_item = None
    group_name = ""
    group_icon = ICON_APPLICATION
    group_pos = None
    plugin_id = -1
    plugin_ui = False
    plugin_inline = False
    ports_data = []
    conns_data = []

    # Step 1 - Store all Item data
    for group in canvas.group_list:
        if group.group_id == group_id:
            if not group.split:
                if canvas.debug:
                    print("PatchCanvas::joinGroup(%i) - group is not split" % group_id)
                return

            item = group.widgets[0]
            s_item = group.widgets[1]
            group_name = group.group_name
            group_icon = group.icon
            group_pos = item.pos()
            plugin_id = group.plugin_id
            plugin_ui = group.plugin_ui
            plugin_inline = group.plugin_inline
            break

    # FIXME
    if not (item and s_item):
        qCritical("PatchCanvas::joinGroup(%i) - unable to find groups to join" % group_id)
        return

    port_list_ids = list(item.getPortList())
    port_list_idss = s_item.getPortList()

    for port_id in port_list_idss:
        if port_id not in port_list_ids:
            port_list_ids.append(port_id)

    for port in canvas.port_list:
        if port.group_id == group_id and port.port_id in port_list_ids:
            port_dict = port_dict_t()
            port_dict.group_id = port.group_id
            port_dict.port_id = port.port_id
            port_dict.port_name = port.port_name
            port_dict.port_mode = port.port_mode
            port_dict.port_type = port.port_type
            port_dict.is_alternate = port.is_alternate
            port_dict.widget = None
            ports_data.append(port_dict)

    for connection in canvas.connection_list:
        if (connection.group_in_id == group_id and connection.port_in_id in port_list_ids) or \
           (connection.group_out_id == group_id and connection.port_out_id in port_list_ids):
            connection_dict = connection_dict_t()
            connection_dict.connection_id = connection.connection_id
            connection_dict.group_in_id = connection.group_in_id
            connection_dict.port_in_id = connection.port_in_id
            connection_dict.group_out_id = connection.group_out_id
            connection_dict.port_out_id = connection.port_out_id
            connection_dict.widget = None
            conns_data.append(connection_dict)

    # Step 2 - Remove Item and Children
    for conn in conns_data:
        disconnectPorts(conn.connection_id)

    for port_id in port_list_ids:
        removePort(group_id, port_id)

    removeGroup(group_id)

    # Step 3 - Re-create Item, now together
    group = addGroup(group_id, group_name, SPLIT_NO, group_icon)

    if plugin_id >= 0:
        setGroupAsPlugin(group_id, plugin_id, plugin_ui, plugin_inline)

    for port in ports_data:
        addPort(group_id, port.port_id, port.port_name, port.port_mode, port.port_type, port.is_alternate)

    for conn in conns_data:
        connectPorts(conn.connection_id, conn.group_out_id, conn.port_out_id, conn.group_in_id, conn.port_in_id, True)

    if group is not None:
        group.widgets[0].blockSignals(True)
        group.widgets[0].setPos(group_pos)
        group.widgets[0].checkItemPos()
        group.widgets[0].blockSignals(False)
        valueStr = "%i:%i:%i:%i" % (group_pos.x(), group_pos.y(), 0, 0)
        CanvasCallback(ACTION_GROUP_POSITION, group_id, 0, valueStr)

    QTimer.singleShot(0, canvas.scene.update)

# ------------------------------------------------------------------------------------------------------------

def getGroupPos(group_id, port_mode=PORT_MODE_OUTPUT):
    if canvas.debug:
        print("PatchCanvas::getGroupPos(%i, %s)" % (group_id, port_mode2str(port_mode)))

    for group in canvas.group_list:
        if group.group_id == group_id:
            return group.widgets[1 if (group.split and port_mode == PORT_MODE_INPUT) else 0].pos()

    qCritical("PatchCanvas::getGroupPos(%i, %s) - unable to find group" % (group_id, port_mode2str(port_mode)))
    return QPointF(0, 0)

def saveGroupPositions():
    if canvas.debug:
        print("PatchCanvas::getGroupPositions()")

    ret = []

    for group in canvas.group_list:
        if group.split:
            pos1 = group.widgets[0].pos()
            pos2 = group.widgets[1].pos()
        else:
            pos1 = group.widgets[0].pos()
            pos2 = QPointF(0, 0)

        ret.append({
            "name" : group.group_name,
            "pos1x": pos1.x(),
            "pos1y": pos1.y(),
            "pos2x": pos2.x(),
            "pos2y": pos2.y(),
            "split": group.split,
        })

    return ret

def restoreGroupPositions(dataList):
    if canvas.debug:
        print("PatchCanvas::restoreGroupPositions(...)")

    mapping = {}

    for group in canvas.group_list:
        mapping[group.group_name] = group

    for data in dataList:
        name = data['name']
        group = mapping.get(name, None)

        if group is None:
            continue

        group.widgets[0].blockSignals(True)
        group.widgets[0].setPos(data['pos1x'], data['pos1y'])
        group.widgets[0].blockSignals(False)

        if group.split and group.widgets[1]:
            group.widgets[1].blockSignals(True)
            group.widgets[1].setPos(data['pos2x'], data['pos2y'])
            group.widgets[1].blockSignals(False)

def setGroupPos(group_id, group_pos_x, group_pos_y):
    setGroupPosFull(group_id, group_pos_x, group_pos_y, group_pos_x, group_pos_y)

def setGroupPosFull(group_id, group_pos_x_o, group_pos_y_o, group_pos_x_i, group_pos_y_i):
    if canvas.debug:
        print("PatchCanvas::setGroupPos(%i, %i, %i, %i, %i)" % (
              group_id, group_pos_x_o, group_pos_y_o, group_pos_x_i, group_pos_y_i))

    for group in canvas.group_list:
        if group.group_id == group_id:
            group.widgets[0].blockSignals(True)
            group.widgets[0].setPos(group_pos_x_o, group_pos_y_o)
            group.widgets[0].checkItemPos()
            group.widgets[0].blockSignals(False)

            if group.split and group.widgets[1]:
                group.widgets[1].blockSignals(True)
                group.widgets[1].setPos(group_pos_x_i, group_pos_y_i)
                group.widgets[1].checkItemPos()
                group.widgets[1].blockSignals(False)

            QTimer.singleShot(0, canvas.scene.update)
            return

    qCritical("PatchCanvas::setGroupPos(%i, %i, %i, %i, %i) - unable to find group to reposition" % (
              group_id, group_pos_x_o, group_pos_y_o, group_pos_x_i, group_pos_y_i))

# ------------------------------------------------------------------------------------------------------------

def setGroupIcon(group_id, icon):
    if canvas.debug:
        print("PatchCanvas::setGroupIcon(%i, %s)" % (group_id, icon2str(icon)))

    for group in canvas.group_list:
        if group.group_id == group_id:
            group.icon = icon
            group.widgets[0].setIcon(icon)

            if group.split and group.widgets[1]:
                group.widgets[1].setIcon(icon)

            QTimer.singleShot(0, canvas.scene.update)
            return

    qCritical("PatchCanvas::setGroupIcon(%i, %s) - unable to find group to change icon" % (group_id, icon2str(icon)))

def setGroupAsPlugin(group_id, plugin_id, hasUI, hasInlineDisplay):
    if canvas.debug:
        print("PatchCanvas::setGroupAsPlugin(%i, %i, %s, %s)" % (
              group_id, plugin_id, bool2str(hasUI), bool2str(hasInlineDisplay)))

    for group in canvas.group_list:
        if group.group_id == group_id:
            group.plugin_id = plugin_id
            group.plugin_ui = hasUI
            group.plugin_inline = hasInlineDisplay
            group.widgets[0].setAsPlugin(plugin_id, hasUI, hasInlineDisplay)

            if group.split and group.widgets[1]:
                group.widgets[1].setAsPlugin(plugin_id, hasUI, hasInlineDisplay)

            canvas.group_plugin_map[plugin_id] = group
            return

    qCritical("PatchCanvas::setGroupAsPlugin(%i, %i, %s, %s) - unable to find group to set as plugin" % (
              group_id, plugin_id, bool2str(hasUI), bool2str(hasInlineDisplay)))

# ------------------------------------------------------------------------------------------------------------

def focusGroupUsingPluginId(plugin_id):
    if canvas.debug:
        print("PatchCanvas::focusGroupUsingPluginId(%i)" % (plugin_id,))

    if plugin_id < 0 or plugin_id >= MAX_PLUGIN_ID_ALLOWED:
        return False

    for group in canvas.group_list:
        if group.plugin_id == plugin_id:
            item = group.widgets[0]
            canvas.scene.clearSelection()
            canvas.scene.getView().centerOn(item)
            item.setSelected(True)
            return True

def focusGroupUsingGroupName(group_name):
    if canvas.debug:
        print("PatchCanvas::focusGroupUsingGroupName(%s)" % (group_name,))

    for group in canvas.group_list:
        if group.group_name == group_name:
            item = group.widgets[0]
            canvas.scene.clearSelection()
            canvas.scene.getView().centerOn(item)
            item.setSelected(True)
            return True

# ------------------------------------------------------------------------------------------------------------

def addPort(group_id, port_id, port_name, port_mode, port_type, is_alternate=False):
    if canvas.debug:
        print("PatchCanvas::addPort(%i, %i, %s, %s, %s, %s)" % (
              group_id, port_id, port_name.encode(),
              port_mode2str(port_mode), port_type2str(port_type), bool2str(is_alternate)))

    for port in canvas.port_list:
        if port.group_id == group_id and port.port_id == port_id:
            qWarning("PatchCanvas::addPort(%i, %i, %s, %s, %s) - port already exists" % (
                     group_id, port_id, port_name.encode(), port_mode2str(port_mode), port_type2str(port_type)))
            return

    box_widget = None
    port_widget = None

    for group in canvas.group_list:
        if group.group_id == group_id:
            if group.split and group.widgets[0].getSplitMode() != port_mode and group.widgets[1]:
                n = 1
            else:
                n = 0
            box_widget = group.widgets[n]
            port_widget = box_widget.addPortFromGroup(port_id, port_mode, port_type, port_name, is_alternate)
            break

    if not (box_widget and port_widget):
        qCritical("PatchCanvas::addPort(%i, %i, %s, %s, %s) - Unable to find parent group" % (
                  group_id, port_id, port_name.encode(), port_mode2str(port_mode), port_type2str(port_type)))
        return

    port_dict = port_dict_t()
    port_dict.group_id = group_id
    port_dict.port_id = port_id
    port_dict.port_name = port_name
    port_dict.port_mode = port_mode
    port_dict.port_type = port_type
    port_dict.is_alternate = is_alternate
    port_dict.widget = port_widget
    canvas.port_list.append(port_dict)

    box_widget.updatePositions()

    if options.eyecandy == EYECANDY_FULL:
        CanvasItemFX(port_widget, True, False)
        return

    QTimer.singleShot(0, canvas.scene.update)

def removePort(group_id, port_id):
    if canvas.debug:
        print("PatchCanvas::removePort(%i, %i)" % (group_id, port_id))

    for port in canvas.port_list:
        if port.group_id == group_id and port.port_id == port_id:
            item = port.widget
            try:
                pitem = item.parentItem()
                canvas.scene.removeItem(item)
            except RuntimeError:
                pass
            else:
                pitem.removePortFromGroup(port_id)
            canvas.port_list.remove(port)
            del item

            QTimer.singleShot(0, canvas.scene.update)
            return

    qCritical("PatchCanvas::removePort(%i, %i) - Unable to find port to remove" % (group_id, port_id))

def renamePort(group_id, port_id, new_port_name):
    if canvas.debug:
        print("PatchCanvas::renamePort(%i, %i, %s)" % (group_id, port_id, new_port_name.encode()))

    for port in canvas.port_list:
        if port.group_id == group_id and port.port_id == port_id:
            port.port_name = new_port_name
            port.widget.setPortName(new_port_name)
            port.widget.parentItem().updatePositions()

            QTimer.singleShot(0, canvas.scene.update)
            return

    qCritical("PatchCanvas::renamePort(%i, %i, %s) - Unable to find port to rename" % (
              group_id, port_id, new_port_name.encode()))

def connectPorts(connection_id, group_out_id, port_out_id, group_in_id, port_in_id, fromSplitOrJoin = False):
    if canvas.last_connection_id >= connection_id and not fromSplitOrJoin:
        print("PatchCanvas::connectPorts(%i, %i, %i, %i, %i) - invalid connection id received (last: %i)" % (
              connection_id, group_out_id, port_out_id, group_in_id, port_in_id, canvas.last_connection_id))
        return

    canvas.last_connection_id = connection_id

    if canvas.debug:
        print("PatchCanvas::connectPorts(%i, %i, %i, %i, %i)" % (
              connection_id, group_out_id, port_out_id, group_in_id, port_in_id))

    port_out = None
    port_in = None
    port_out_parent = None
    port_in_parent = None

    for port in canvas.port_list:
        if port.group_id == group_out_id and port.port_id == port_out_id:
            port_out = port.widget
            port_out_parent = port_out.parentItem()
        elif port.group_id == group_in_id and port.port_id == port_in_id:
            port_in = port.widget
            port_in_parent = port_in.parentItem()

    # FIXME
    if not (port_out and port_in):
        qCritical("PatchCanvas::connectPorts(%i, %i, %i, %i, %i) - unable to find ports to connect" % (
                  connection_id, group_out_id, port_out_id, group_in_id, port_in_id))
        return

    connection_dict = connection_dict_t()
    connection_dict.connection_id = connection_id
    connection_dict.group_in_id = group_in_id
    connection_dict.port_in_id = port_in_id
    connection_dict.group_out_id = group_out_id
    connection_dict.port_out_id = port_out_id

    if options.use_bezier_lines:
        connection_dict.widget = CanvasBezierLine(port_out, port_in, None)
    else:
        connection_dict.widget = CanvasLine(port_out, port_in, None)

    canvas.scene.addItem(connection_dict.widget)

    port_out_parent.addLineFromGroup(connection_dict.widget, connection_id)
    port_in_parent.addLineFromGroup(connection_dict.widget, connection_id)

    canvas.last_z_value += 1
    port_out_parent.setZValue(canvas.last_z_value)
    port_in_parent.setZValue(canvas.last_z_value)

    canvas.last_z_value += 1
    connection_dict.widget.setZValue(canvas.last_z_value)

    canvas.connection_list.append(connection_dict)

    if options.eyecandy == EYECANDY_FULL:
        item = connection_dict.widget
        CanvasItemFX(item, True, False)
        return

    QTimer.singleShot(0, canvas.scene.update)

def disconnectPorts(connection_id):
    if canvas.debug:
        print("PatchCanvas::disconnectPorts(%i)" % connection_id)

    line = None
    item1 = None
    item2 = None
    group1id = port1id = 0
    group2id = port2id = 0

    for connection in canvas.connection_list:
        if connection.connection_id == connection_id:
            group1id = connection.group_out_id
            group2id = connection.group_in_id
            port1id = connection.port_out_id
            port2id = connection.port_in_id
            line = connection.widget
            canvas.connection_list.remove(connection)
            break

    if not line:
        qCritical("PatchCanvas::disconnectPorts(%i) - unable to find connection ports" % connection_id)
        return

    for port in canvas.port_list:
        if port.group_id == group1id and port.port_id == port1id:
            item1 = port.widget
            break

    if not item1:
        qCritical("PatchCanvas::disconnectPorts(%i) - unable to find output port" % connection_id)
        return

    for port in canvas.port_list:
        if port.group_id == group2id and port.port_id == port2id:
            item2 = port.widget
            break

    if not item2:
        qCritical("PatchCanvas::disconnectPorts(%i) - unable to find input port" % connection_id)
        return

    item1p = item1.parentItem()
    item2p = item2.parentItem()
    if item1p:
        item1p.removeLineFromGroup(connection_id)
    if item2p:
        item2p.removeLineFromGroup(connection_id)

    if options.eyecandy == EYECANDY_FULL:
        CanvasItemFX(line, False, True)
        return

    canvas.scene.removeItem(line)
    del line

    QTimer.singleShot(0, canvas.scene.update)

# ------------------------------------------------------------------------------------------------------------

def arrange():
    if canvas.debug:
        print("PatchCanvas::arrange()")

# ------------------------------------------------------------------------------------------------------------

def updateZValues():
    if canvas.debug:
        print("PatchCanvas::updateZValues()")

    for group in canvas.group_list:
        group.widgets[0].resetLinesZValue()

        if group.split and group.widgets[1]:
            group.widgets[1].resetLinesZValue()

# ------------------------------------------------------------------------------------------------------------

def redrawPluginGroup(plugin_id):
    group = canvas.group_plugin_map.get(plugin_id, None)

    if group is None:
        #qCritical("PatchCanvas::redrawPluginGroup(%i) - unable to find group" % plugin_id)
        return

    group.widgets[0].redrawInlineDisplay()

    if group.split and group.widgets[1]:
        group.widgets[1].redrawInlineDisplay()

def handlePluginRemoved(plugin_id):
    if canvas.debug:
        print("PatchCanvas::handlePluginRemoved(%i)" % plugin_id)

    canvas.scene.clearSelection()

    group = canvas.group_plugin_map.pop(plugin_id, None)

    if group is not None:
        group.plugin_id = -1
        group.plugin_ui = False
        group.plugin_inline = False
        group.widgets[0].removeAsPlugin()

        if group.split and group.widgets[1]:
            group.widgets[1].removeAsPlugin()

    for group in canvas.group_list:
        if group.plugin_id < plugin_id or group.plugin_id > MAX_PLUGIN_ID_ALLOWED:
            continue

        group.plugin_id -= 1
        group.widgets[0].m_plugin_id -= 1

        if group.split and group.widgets[1]:
            group.widgets[1].m_plugin_id -= 1

        canvas.group_plugin_map[plugin_id] = group

def handleAllPluginsRemoved():
    if canvas.debug:
        print("PatchCanvas::handleAllPluginsRemoved()")

    canvas.group_plugin_map = {}

    for group in canvas.group_list:
        if group.plugin_id < 0:
            continue
        if group.plugin_id > MAX_PLUGIN_ID_ALLOWED:
            continue

        group.plugin_id = -1
        group.plugin_ui = False
        group.plugin_inline = False
        group.widgets[0].removeAsPlugin()

        if group.split and group.widgets[1]:
            group.widgets[1].removeAsPlugin()

# ------------------------------------------------------------------------------------------------------------
